searchData={"items":[{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"LearnOpenGL 4.2 - Textures Combined\n\nThis example demonstrates using multiple texture units to combine textures in a single\nfragment shader. It builds on the basic texture concepts from 4.1 and shows how to\nmix two textures together using GLSL's mix() function.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"In the original LearnOpenGL C++ tutorial, this example introduces:\n- Multiple texture units (GL_TEXTURE0, GL_TEXTURE1)\n- Binding textures to different texture units\n- Using multiple sampler2D uniforms in fragment shader\n- The GLSL mix() function for blending textures\n- Setting sampler uniform values to specify texture units\n\nEAGL's framework preserves all these concepts while providing enhanced functionality:\n- **Simplified texture loading**: Uses EAGL's texture helpers with fallback support\n- **Graceful degradation**: Works with or without external image dependencies\n- **Error handling**: Comprehensive error checking and user feedback\n- **Educational value**: Clear documentation of texture unit concepts","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#module-framework-adaptation-notes"},{"type":"module","title":"EAGL vs Original Implementation - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"**Original LearnOpenGL approach:** Manual texture unit management:\n```c++\nglActiveTexture(GL_TEXTURE0);\nglBindTexture(GL_TEXTURE_2D, texture1);\nglActiveTexture(GL_TEXTURE1);\nglBindTexture(GL_TEXTURE_2D, texture2);\nglUniform1i(glGetUniformLocation(shaderProgram, \"texture1\"), 0);\nglUniform1i(glGetUniformLocation(shaderProgram, \"texture2\"), 1);\n```\n\n**EAGL approach:** Texture helpers with explicit texture unit management:\n```elixir\n# Load textures with fallback\n{:ok, texture1_id, _, _} = load_texture_from_file(\"image1.jpg\")\n{:ok, texture2_id, _, _} = create_checkerboard_texture(256, 32)\n\n# Set shader uniforms\nset_uniforms(program, [\n  texture1: 0,  # Texture unit 0\n  texture2: 1   # Texture unit 1\n])\n```","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#module-eagl-vs-original-implementation"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"- **Texture Units**: OpenGL provides multiple texture units (GL_TEXTURE0, GL_TEXTURE1, etc.)\n- **Sampler Uniforms**: Fragment shaders can have multiple sampler2D uniforms\n- **Texture Binding**: Each texture unit can have a different texture bound\n- **Mix Function**: GLSL's mix() function linearly interpolates between two values\n- **Uniform Assignment**: Sampler uniforms need to be set to the correct texture unit number","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#module-key-learning-points"},{"type":"module","title":"Texture Mixing - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"The fragment shader uses GLSL's mix() function:\n```glsl\nFragColor = mix(texture(texture1, TexCoord), texture(texture2, TexCoord), 0.2);\n```\n- First parameter: First texture sample\n- Second parameter: Second texture sample\n- Third parameter: Mix factor (0.2 = 80% first texture, 20% second texture)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#module-texture-mixing"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesCombined.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"LearnOpenGL 4.3 - Textures Exercise 1\n\nThis exercise demonstrates texture coordinate manipulation to show only part of a texture\nor flip its orientation. It's commonly used to show the center pixels of a texture or\nto flip textures horizontally/vertically.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"This example is based on the original LearnOpenGL C++ tutorial exercises:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#module-original-c-source"},{"type":"module","title":"Exercise Focus - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"Common texture coordinate exercises include:\n- **Center Cropping**: Using texture coordinates like (0.25, 0.25) to (0.75, 0.75)\n- **Texture Flipping**: Reversing texture coordinates to flip images\n- **Coordinate Scaling**: Using coordinates > 1.0 to see texture wrapping effects\n- **Pixel Visibility**: Using GL_NEAREST filtering to see individual pixels","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#module-exercise-focus"},{"type":"module","title":"EAGL Implementation - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"This implementation demonstrates center cropping - showing only the central portion\nof the texture by using texture coordinates that map to the center 50% of the image:\n\n```elixir\n# Instead of full texture coordinates (0.0 to 1.0)\n# Use center coordinates (0.25 to 0.75)\n@vertices [\n  # positions      # colors       # texture coords (center crop)\n   0.5,  0.5, 0.0, 1.0, 0.0, 0.0, 0.75, 0.75,  # top right\n   0.5, -0.5, 0.0, 0.0, 1.0, 0.0, 0.75, 0.25,  # bottom right\n  -0.5, -0.5, 0.0, 0.0, 0.0, 1.0, 0.25, 0.25,  # bottom left\n  -0.5,  0.5, 0.0, 1.0, 1.0, 0.0, 0.25, 0.75   # top left\n]\n```","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#module-eagl-implementation"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"- **Texture Coordinates**: Don't have to span full 0.0 to 1.0 range\n- **Partial Sampling**: Can show only portions of textures\n- **Coordinate Mapping**: How vertex positions map to texture coordinates\n- **Filtering Effects**: How GL_NEAREST vs GL_LINEAR affects small texture regions\n- **Creative Control**: Texture coordinates provide artistic flexibility","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#module-key-learning-points"},{"type":"module","title":"Visual Effect - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"This exercise shows only the center portion of the texture, effectively cropping\nthe image and scaling it to fill the rectangle. With GL_NEAREST filtering,\nindividual pixels become more visible.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#module-visual-effect"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise1.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"LearnOpenGL 4.4 - Textures Exercise 2\n\nThis exercise demonstrates texture wrapping modes by using texture coordinates\noutside the 0.0 to 1.0 range. It shows how different wrapping modes (repeat,\nmirrored repeat, clamp to edge, clamp to border) affect texture appearance.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"This example is based on the original LearnOpenGL C++ tutorial exercises:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-original-c-source"},{"type":"module","title":"Exercise Focus - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"This exercise demonstrates:\n- **Texture Coordinates > 1.0**: Using coordinates like 0.0 to 2.0 instead of 0.0 to 1.0\n- **Texture Wrapping Modes**: Different ways OpenGL handles out-of-range coordinates\n- **Visual Patterns**: How wrapping creates repeating or clamped patterns\n- **Border Colors**: Setting custom colors for border clamping mode","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-exercise-focus"},{"type":"module","title":"Texture Wrapping Modes - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"- **GL_REPEAT**: Default - texture repeats infinitely (tiles)\n- **GL_MIRRORED_REPEAT**: Texture repeats but alternates mirrored\n- **GL_CLAMP_TO_EDGE**: Coordinates clamped to 0.0-1.0, edges stretched\n- **GL_CLAMP_TO_BORDER**: Out-of-range areas filled with border color","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-texture-wrapping-modes"},{"type":"module","title":"EAGL Implementation - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"This implementation uses texture coordinates from 0.0 to 2.0, causing the texture\nto repeat twice in each direction with GL_REPEAT mode:\n\n```elixir\n@vertices [\n  # positions      # colors       # texture coords (0.0 to 2.0)\n   0.5,  0.5, 0.0, 1.0, 0.0, 0.0, 2.0, 2.0,  # top right\n   0.5, -0.5, 0.0, 0.0, 1.0, 0.0, 2.0, 0.0,  # bottom right\n  -0.5, -0.5, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0,  # bottom left\n  -0.5,  0.5, 0.0, 1.0, 1.0, 0.0, 0.0, 2.0   # top left\n]\n```","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-eagl-implementation"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"- **Texture Wrapping**: How OpenGL handles coordinates outside 0.0-1.0\n- **Tiling Effects**: Creating repeating patterns with textures\n- **Border Control**: Using custom border colors for specific effects\n- **Pattern Variety**: Different wrapping modes create different visual patterns","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-key-learning-points"},{"type":"module","title":"Visual Effect - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"With GL_REPEAT mode and 2.0 coordinates, you'll see the texture repeated\n4 times (2x2 grid). This demonstrates how texture wrapping enables tiling\npatterns for surfaces like floors, walls, or repeating backgrounds.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-visual-effect"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise2.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3","doc":"LearnOpenGL 4.5 - Textures Exercise 3\n\nThis exercise demonstrates texture flipping by manipulating texture coordinates.\nCommon variations include horizontal flipping, vertical flipping, or rotating\ntextures through coordinate transformation.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3","doc":"This example is based on the original LearnOpenGL C++ tutorial exercises:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html#module-original-c-source"},{"type":"module","title":"Exercise Focus - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3","doc":"This exercise demonstrates:\n- **Texture Flipping**: Inverting texture coordinates to flip images\n- **Coordinate Manipulation**: How to transform texture space\n- **Visual Transformations**: Creating mirrored or rotated effects\n- **Creative Control**: Using coordinates for artistic effects","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html#module-exercise-focus"},{"type":"module","title":"EAGL Implementation - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3","doc":"This implementation demonstrates horizontal flipping by reversing the S (horizontal)\ntexture coordinates:\n\n```elixir\n@vertices [\n  # positions      # colors       # texture coords (horizontally flipped)\n   0.5,  0.5, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0,  # top right -> top left\n   0.5, -0.5, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0,  # bottom right -> bottom left\n  -0.5, -0.5, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0,  # bottom left -> bottom right\n  -0.5,  0.5, 0.0, 1.0, 1.0, 0.0, 1.0, 1.0   # top left -> top right\n]\n```","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html#module-eagl-implementation"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3","doc":"- **Coordinate Transformation**: How to manipulate texture coordinates\n- **Flipping Techniques**: Horizontal and vertical texture flipping\n- **Visual Effects**: Creating mirrored or transformed appearances\n- **Creative Applications**: Using coordinate manipulation for artistic control","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html#module-key-learning-points"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise3.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"LearnOpenGL 4.6 - Textures Exercise 4\n\nThis exercise demonstrates dynamic texture mixing with animated mix ratios.\nInstead of a fixed mix value, the blend between textures changes over time,\ncreating an animated transition effect.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"This example is based on the original LearnOpenGL C++ tutorial exercises:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#module-original-c-source"},{"type":"module","title":"Exercise Focus - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"This exercise demonstrates:\n- **Dynamic Uniforms**: Updating uniform values each frame\n- **Time-based Animation**: Using time to create smooth transitions\n- **Interactive Effects**: Creating animated visual effects with textures\n- **Mix Function Control**: Real-time control of texture blending","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#module-exercise-focus"},{"type":"module","title":"EAGL Implementation - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"This implementation uses time-based animation to cycle the mix ratio between\ntwo textures, creating a smooth transition effect:\n\n```glsl\n# In the fragment shader:\nmix_factor = sin(time * 2.0) * 0.5 + 0.5  # Oscillates between 0.0 and 1.0\nFragColor = mix(texture1, texture2, mix_factor)\n```","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#module-eagl-implementation"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"- **Dynamic Uniforms**: How to update shader uniforms each frame\n- **Time Animation**: Using time to create smooth, cyclic animations\n- **Visual Effects**: Creating engaging animated transitions\n- **Interactive Graphics**: Real-time parameter control in shaders","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#module-key-learning-points"},{"type":"module","title":"Visual Effect - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"The example shows two textures that smoothly blend back and forth over time.\nThe mix ratio oscillates between 0.0 (first texture only) and 1.0 (second\ntexture only), creating a rhythmic fade effect.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#module-visual-effect"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.TexturesExercise4.html#run_example/0"},{"type":"behaviour","title":"EAGL.WindowBehaviour","doc":"Behaviour for OpenGL window applications.\n\nDefines callbacks for OpenGL window lifecycle management with\nautomatic context setup and event handling.","ref":"EAGL.WindowBehaviour.html"},{"type":"behaviour","title":"Usage - EAGL.WindowBehaviour","doc":"defmodule MyApp do\n      use EAGL.Window\n\n      @impl true\n      def setup do\n        # Initialize shaders, load models, etc.\n        {:ok, initial_state}\n      end\n\n      @impl true\n      def render(width, height, state) do\n        # Clear and render your scene\n        :gl.clearColor(0.2, 0.3, 0.3, 1.0)\n        :gl.clear(@gl_color_buffer_bit)\n        # ... render content\n        :ok\n      end\n\n      @impl true\n      def cleanup(state) do\n        # Clean up resources\n        :ok\n      end\n    end\n\n    # Run the application\n    EAGL.Window.run(MyApp, \"My OpenGL App\")","ref":"EAGL.WindowBehaviour.html#module-usage"},{"type":"callback","title":"EAGL.WindowBehaviour.cleanup/1","doc":"Called when the window is being closed.\nShould clean up resources in state.","ref":"EAGL.WindowBehaviour.html#c:cleanup/1"},{"type":"callback","title":"EAGL.WindowBehaviour.handle_event/2","doc":"Called when an event occurs (keyboard, mouse, etc.).\nThis callback is optional. If not implemented, events are ignored.\nShould return {:ok, new_state} to continue with updated state.","ref":"EAGL.WindowBehaviour.html#c:handle_event/2"},{"type":"callback","title":"EAGL.WindowBehaviour.render/3","doc":"Called when the window needs to be rendered.\nThe OpenGL context is already current and the viewport is set.","ref":"EAGL.WindowBehaviour.html#c:render/3"},{"type":"callback","title":"EAGL.WindowBehaviour.setup/0","doc":"Called when the OpenGL context is ready and shaders should be created.\nShould return {:ok, state} or {:error, reason}","ref":"EAGL.WindowBehaviour.html#c:setup/0"},{"type":"task","title":"mix eagl.test","doc":"Runs EAGL tests excluding interactive examples that require user input.\n\nThis prevents tests from hanging while waiting for ENTER key presses.","ref":"Mix.Tasks.Eagl.Test.html"},{"type":"task","title":"Examples - mix eagl.test","doc":"mix eagl.test\n    mix eagl.test --verbose","ref":"Mix.Tasks.Eagl.Test.html#module-examples"},{"type":"function","title":"Mix.Tasks.Eagl.Test.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Eagl.Test.html#run/1"},{"type":"module","title":"WX.Const","doc":"wxWidgets constants for GUI and OpenGL canvas management.\nUse this module to inject wxWidgets constants as module attributes.","ref":"WX.Const.html"},{"type":"module","title":"Usage - WX.Const","doc":"defmodule MyModule do\n      use WX.Const\n\n      # Now you can use @wx_vertical, @wx_gl_rgba, @wx_expand, etc.\n    end","ref":"WX.Const.html#module-usage"},{"type":"module","title":"Categories - WX.Const","doc":"This module provides constants for:\n- OpenGL canvas attributes (WX_GL_*)\n- Layout directions and sizer flags\n- Window styles and background styles\n- Event types and window management","ref":"WX.Const.html#module-categories"},{"type":"module","title":"EAGL.Buffer","doc":"OpenGL buffer and vertex array object management.\n\nProvides Wings3D-inspired helper functions for common VAO/VBO/EBO operations\nwith meaningful abstractions for vertex attribute setup.\n\nThe module includes type-safe helpers for common vertex attributes like\nposition, color, texture coordinates, and normals.","ref":"EAGL.Buffer.html"},{"type":"module","title":"Original Source - EAGL.Buffer","doc":"Buffer management patterns and helper functions are inspired by Wings3D's\n`wings_gl.erl` module:","ref":"EAGL.Buffer.html#module-original-source"},{"type":"module","title":"Usage - EAGL.Buffer","doc":"import EAGL.Buffer\n\n    # Simple position-only vertices\n    vertices = [-0.5, -0.5, 0.0, 0.5, -0.5, 0.0, 0.0, 0.5, 0.0]\n    {vao, vbo} = create_position_array(vertices)\n\n    # Custom vertex attributes with type-safe configuration\n    attributes = [\n      vertex_attribute(location: 0, size: 3, type: :float, stride: 24, offset: 0),   # position\n      vertex_attribute(location: 1, size: 3, type: :float, stride: 24, offset: 12)  # color\n    ]\n    {vao, vbo} = create_vertex_array(vertices, attributes)\n\n    # Or use standard attribute helpers\n    attributes = [\n      position_attribute(stride: 24, offset: 0),\n      color_attribute(stride: 24, offset: 12)\n    ]\n    {vao, vbo} = create_vertex_array(vertices, attributes)\n\n    # Indexed geometry (rectangles, models)\n    vertices = [0.5, 0.5, 0.0, 0.5, -0.5, 0.0, -0.5, -0.5, 0.0, -0.5, 0.5, 0.0]\n    indices = [0, 1, 3, 1, 2, 3]\n    {vao, vbo, ebo} = create_indexed_position_array(vertices, indices)\n\n    # Use OpenGL directly for rendering\n    :gl.bindVertexArray(vao)\n    :gl.drawElements(@gl_triangles, 6, @gl_unsigned_int, 0)\n\n    # Clean up\n    delete_vertex_array(vao, vbo)","ref":"EAGL.Buffer.html#module-usage"},{"type":"type","title":"EAGL.Buffer.buffer_id/0","doc":"","ref":"EAGL.Buffer.html#t:buffer_id/0"},{"type":"type","title":"EAGL.Buffer.buffer_usage/0","doc":"","ref":"EAGL.Buffer.html#t:buffer_usage/0"},{"type":"function","title":"EAGL.Buffer.color_attribute/1","doc":"Creates a standard colour attribute configuration (location 1, vec3, float).","ref":"EAGL.Buffer.html#color_attribute/1"},{"type":"function","title":"EAGL.Buffer.create_indexed_array/4","doc":"Creates a VAO with VBO and EBO for indexed geometry.\nReturns {vao, vbo, ebo} tuple.","ref":"EAGL.Buffer.html#create_indexed_array/4"},{"type":"function","title":"Parameters - EAGL.Buffer.create_indexed_array/4","doc":"- vertices: List of floats representing vertex data\n- indices: List of integers representing vertex indices\n- attributes: List of vertex_attribute structs\n- opts: Options for buffer creation","ref":"EAGL.Buffer.html#create_indexed_array/4-parameters"},{"type":"function","title":"Options - EAGL.Buffer.create_indexed_array/4","doc":"- `usage`: Buffer usage pattern (default: :static_draw)","ref":"EAGL.Buffer.html#create_indexed_array/4-options"},{"type":"function","title":"Example - EAGL.Buffer.create_indexed_array/4","doc":"vertices = [0.5, 0.5, 0.0, 0.5, -0.5, 0.0, -0.5, -0.5, 0.0, -0.5, 0.5, 0.0]\n    indices = [0, 1, 3, 1, 2, 3]\n\n    attributes = [position_attribute()]\n    {vao, vbo, ebo} = create_indexed_array(vertices, indices, attributes)","ref":"EAGL.Buffer.html#create_indexed_array/4-example"},{"type":"function","title":"EAGL.Buffer.create_indexed_position_array/2","doc":"Creates a simple indexed VAO with position-only vertices (3 floats per vertex).\nConvenience wrapper for indexed geometry with positions only.","ref":"EAGL.Buffer.html#create_indexed_position_array/2"},{"type":"function","title":"Example - EAGL.Buffer.create_indexed_position_array/2","doc":"vertices = [0.5, 0.5, 0.0, 0.5, -0.5, 0.0, -0.5, -0.5, 0.0, -0.5, 0.5, 0.0]\n    indices = [0, 1, 3, 1, 2, 3]  # Two triangles forming a rectangle\n    {vao, vbo, ebo} = create_indexed_position_array(vertices, indices)","ref":"EAGL.Buffer.html#create_indexed_position_array/2-example"},{"type":"function","title":"EAGL.Buffer.create_position_array/1","doc":"Creates a simple VAO with position-only vertices (3 floats per vertex).\nConvenience wrapper for the most common case.","ref":"EAGL.Buffer.html#create_position_array/1"},{"type":"function","title":"Example - EAGL.Buffer.create_position_array/1","doc":"vertices = [-0.5, -0.5, 0.0, 0.5, -0.5, 0.0, 0.0, 0.5, 0.0]\n    {vao, vbo} = create_position_array(vertices)","ref":"EAGL.Buffer.html#create_position_array/1-example"},{"type":"function","title":"EAGL.Buffer.create_vertex_array/3","doc":"Creates a VAO with a single VBO containing vertex data.\nReturns {vao, vbo} tuple.","ref":"EAGL.Buffer.html#create_vertex_array/3"},{"type":"function","title":"Parameters - EAGL.Buffer.create_vertex_array/3","doc":"- vertices: List of floats representing vertex data\n- attributes: List of vertex_attribute structs\n- opts: Options for buffer creation","ref":"EAGL.Buffer.html#create_vertex_array/3-parameters"},{"type":"function","title":"Options - EAGL.Buffer.create_vertex_array/3","doc":"- `usage`: Buffer usage pattern (default: :static_draw)","ref":"EAGL.Buffer.html#create_vertex_array/3-options"},{"type":"function","title":"Example - EAGL.Buffer.create_vertex_array/3","doc":"vertices = [-0.5, -0.5, 0.0, 0.5, -0.5, 0.0, 0.0, 0.5, 0.0]\n\n    # Type-safe approach\n    attributes = [\n      position_attribute(),\n      color_attribute(stride: 24, offset: 12)\n    ]\n    {vao, vbo} = create_vertex_array(vertices, attributes)","ref":"EAGL.Buffer.html#create_vertex_array/3-example"},{"type":"function","title":"EAGL.Buffer.delete_indexed_array/3","doc":"Deletes a VAO and its associated VBO and EBO.","ref":"EAGL.Buffer.html#delete_indexed_array/3"},{"type":"function","title":"EAGL.Buffer.delete_vertex_array/2","doc":"Deletes a VAO and its associated VBOs.","ref":"EAGL.Buffer.html#delete_vertex_array/2"},{"type":"function","title":"EAGL.Buffer.indices_to_binary/1","doc":"Converts a list of indices to binary format for OpenGL.","ref":"EAGL.Buffer.html#indices_to_binary/1"},{"type":"function","title":"EAGL.Buffer.normal_attribute/1","doc":"Creates a standard normal attribute configuration (location 3, vec3, float).","ref":"EAGL.Buffer.html#normal_attribute/1"},{"type":"function","title":"EAGL.Buffer.position_attribute/1","doc":"Creates a standard position attribute configuration (location 0, vec3, float).","ref":"EAGL.Buffer.html#position_attribute/1"},{"type":"function","title":"EAGL.Buffer.texture_coordinate_attribute/1","doc":"Creates a standard texture coordinate attribute configuration (location 2, vec2, float).","ref":"EAGL.Buffer.html#texture_coordinate_attribute/1"},{"type":"type","title":"EAGL.Buffer.vao_id/0","doc":"","ref":"EAGL.Buffer.html#t:vao_id/0"},{"type":"type","title":"EAGL.Buffer.vertex_attribute/0","doc":"","ref":"EAGL.Buffer.html#t:vertex_attribute/0"},{"type":"function","title":"EAGL.Buffer.vertex_attribute/1","doc":"Creates a vertex attribute configuration with type safety and sensible defaults.","ref":"EAGL.Buffer.html#vertex_attribute/1"},{"type":"function","title":"Options - EAGL.Buffer.vertex_attribute/1","doc":"- `location`: Attribute location in shader (required)\n- `size`: Number of components (1-4, required)\n- `type`: Data type (default: :float)\n- `normalized`: Whether to normalize fixed-point data (default: false)\n- `stride`: Byte offset between consecutive attributes (default: 0 for tightly packed)\n- `offset`: Byte offset of first component in buffer (default: 0)","ref":"EAGL.Buffer.html#vertex_attribute/1-options"},{"type":"function","title":"Examples - EAGL.Buffer.vertex_attribute/1","doc":"# Position attribute (vec3)\n    pos_attr = vertex_attribute(location: 0, size: 3)\n\n    # Color attribute with custom stride and offset\n    color_attr = vertex_attribute(location: 1, size: 3, type: :float, stride: 24, offset: 12)\n\n    # Normalized integer attribute\n    normal_attr = vertex_attribute(location: 2, size: 3, type: :byte, normalized: true)","ref":"EAGL.Buffer.html#vertex_attribute/1-examples"},{"type":"type","title":"EAGL.Buffer.vertex_attribute_name/0","doc":"","ref":"EAGL.Buffer.html#t:vertex_attribute_name/0"},{"type":"type","title":"EAGL.Buffer.vertex_attribute_type/0","doc":"","ref":"EAGL.Buffer.html#t:vertex_attribute_type/0"},{"type":"function","title":"EAGL.Buffer.vertex_attributes/1","doc":"Creates a list of vertex attributes with automatic stride and offset calculation.\nThis is a higher-level helper that eliminates the redundancy of manually\nspecifying stride and offset for common interleaved vertex layouts.","ref":"EAGL.Buffer.html#vertex_attributes/1"},{"type":"function","title":"Usage - EAGL.Buffer.vertex_attributes/1","doc":"# List syntax\n    attributes = vertex_attributes([:position, :color])\n\n    # Multiple arguments syntax\n    attributes = vertex_attributes(:position, :color, :texture_coordinate)","ref":"EAGL.Buffer.html#vertex_attributes/1-usage"},{"type":"function","title":"Supported Attribute Types - EAGL.Buffer.vertex_attributes/1","doc":"See `t:vertex_attribute_name/0` for the complete type definition.\n\n- `:position` - 3 floats (x, y, z) at location 0\n- `:color` - 3 floats (r, g, b) at location 1\n- `:texture_coordinate` - 2 floats (s, t) at location 2\n- `:normal` - 3 floats (nx, ny, nz) at location 3","ref":"EAGL.Buffer.html#vertex_attributes/1-supported-attribute-types"},{"type":"function","title":"Examples - EAGL.Buffer.vertex_attributes/1","doc":"# Position and color (stride=24, offsets: 0, 12)\n    {vao, vbo} = create_vertex_array(vertices, vertex_attributes([:position, :color]))\n\n    # Position, color, texture (stride=32, offsets: 0, 12, 24)\n    attributes = vertex_attributes(:position, :color, :texture_coordinate)\n    {vao, vbo} = create_vertex_array(vertices, attributes)\n\nThe function automatically calculates:\n- Stride: Total size of one vertex (sum of all attribute sizes)\n- Offsets: Cumulative byte offset for each attribute\n- Locations: Sequential starting from 0","ref":"EAGL.Buffer.html#vertex_attributes/1-examples"},{"type":"function","title":"EAGL.Buffer.vertex_attributes/2","doc":"","ref":"EAGL.Buffer.html#vertex_attributes/2"},{"type":"function","title":"EAGL.Buffer.vertex_attributes/3","doc":"","ref":"EAGL.Buffer.html#vertex_attributes/3"},{"type":"function","title":"EAGL.Buffer.vertex_attributes/4","doc":"","ref":"EAGL.Buffer.html#vertex_attributes/4"},{"type":"function","title":"EAGL.Buffer.vertices_to_binary/1","doc":"Converts a list of vertex floats to binary format for OpenGL.","ref":"EAGL.Buffer.html#vertices_to_binary/1"},{"type":"module","title":"EAGL.Error","doc":"OpenGL error checking and reporting utilities.\n\nProvides Wings3D-inspired error handling patterns with meaningful\nabstractions for development and debugging.","ref":"EAGL.Error.html"},{"type":"module","title":"Original Source - EAGL.Error","doc":"Error handling patterns are inspired by Wings3D's `wings_gl.erl` module:","ref":"EAGL.Error.html#module-original-source"},{"type":"module","title":"Usage - EAGL.Error","doc":"import EAGL.Error\n\n    # Check for errors with context\n    check(\"After buffer creation\")\n\n    # Get error descriptions\n    error_string(1280)  # \"GL_INVALID_ENUM\"\n\n    # Raise on error (debugging)\n    check!(\"Critical operation\")","ref":"EAGL.Error.html#module-usage"},{"type":"function","title":"EAGL.Error.check/1","doc":"Check for OpenGL errors and print them.\nSimilar to Wings3D's check_error/2 function.","ref":"EAGL.Error.html#check/1"},{"type":"function","title":"Parameters - EAGL.Error.check/1","doc":"- context: String describing the context where the error check is performed","ref":"EAGL.Error.html#check/1-parameters"},{"type":"function","title":"Returns - EAGL.Error.check/1","doc":"- `:ok` if no error\n- `{:error, error_message}` if an error occurred","ref":"EAGL.Error.html#check/1-returns"},{"type":"function","title":"Example - EAGL.Error.check/1","doc":"EAGL.Error.check(\"After buffer creation\")\n    EAGL.Error.check(\"Shader compilation\")","ref":"EAGL.Error.html#check/1-example"},{"type":"function","title":"EAGL.Error.check!/1","doc":"Check for OpenGL errors and raise an exception if one is found.\nUseful for development and debugging.","ref":"EAGL.Error.html#check!/1"},{"type":"function","title":"EAGL.Error.error_string/1","doc":"Get a human-readable string for an OpenGL error code.\nSimilar to Wings3D's error_string/1 function.","ref":"EAGL.Error.html#error_string/1"},{"type":"module","title":"EAGL.Math","doc":"GLM-compatible 3D math library for OpenGL applications.\n\nProvides comprehensive vector, matrix, and quaternion operations with\nOpenGL integration based on the GLM library. All functions work with Erlang's tuple-in-list\nformat required by wx OpenGL bindings.","ref":"EAGL.Math.html"},{"type":"module","title":"Usage - EAGL.Math","doc":"import EAGL.Math\n\n    # Vector operations\n    position = vec3(1.0, 2.0, 3.0)\n    direction = vec3(0.0, 1.0, 0.0)\n    result = vec_add(position, direction)\n    length = vec_length(position)\n\n    # Matrix transformations\n    model = mat4_translate(vec3(5.0, 0.0, 0.0))\n    view = mat4_look_at(\n      vec3(0.0, 0.0, 5.0),  # eye\n      vec3(0.0, 0.0, 0.0),  # target\n      vec3(0.0, 1.0, 0.0)   # up\n    )\n    projection = mat4_perspective(radians(45.0), 16.0/9.0, 0.1, 100.0)\n\n    # Quaternion rotations\n    rotation = quat_from_axis_angle(vec3(0.0, 1.0, 0.0), radians(45.0))\n\n    # All values work directly with EAGL.Shader uniform functions\n    set_uniform(program, \"model\", model)\n    set_uniform(program, \"view\", view)\n    set_uniform(program, \"projection\", projection)","ref":"EAGL.Math.html#module-usage"},{"type":"function","title":"EAGL.Math.abs_val/1","doc":"Absolute value.","ref":"EAGL.Math.html#abs_val/1"},{"type":"function","title":"EAGL.Math.angle_between/2","doc":"Calculate angle between two vectors in radians.","ref":"EAGL.Math.html#angle_between/2"},{"type":"function","title":"EAGL.Math.clamp/3","doc":"Clamp value between min and max.","ref":"EAGL.Math.html#clamp/3"},{"type":"function","title":"EAGL.Math.cross/2","doc":"Compute the cross product of two 3D vectors.","ref":"EAGL.Math.html#cross/2"},{"type":"function","title":"EAGL.Math.degrees/1","doc":"Convert radians to degrees.","ref":"EAGL.Math.html#degrees/1"},{"type":"function","title":"EAGL.Math.distance/2","doc":"Calculate the distance between two points (vectors).","ref":"EAGL.Math.html#distance/2"},{"type":"function","title":"EAGL.Math.distance_squared/2","doc":"Calculate the squared distance between two points (avoids sqrt).","ref":"EAGL.Math.html#distance_squared/2"},{"type":"function","title":"EAGL.Math.dot/2","doc":"Compute the dot product of two vectors (2D, 3D, or 4D).","ref":"EAGL.Math.html#dot/2"},{"type":"function","title":"EAGL.Math.face_normal/3","doc":"Calculate face normal from three vertices (assumes counter-clockwise winding).","ref":"EAGL.Math.html#face_normal/3"},{"type":"function","title":"EAGL.Math.fract/1","doc":"Fractional part of x (x - floor(x)).","ref":"EAGL.Math.html#fract/1"},{"type":"function","title":"EAGL.Math.inverse_sqrt/1","doc":"Fast inverse square root approximation.","ref":"EAGL.Math.html#inverse_sqrt/1"},{"type":"function","title":"EAGL.Math.length_squared/1","doc":"Compute the squared length of a vector (avoids sqrt).","ref":"EAGL.Math.html#length_squared/1"},{"type":"function","title":"EAGL.Math.lerp/3","doc":"Linear interpolation between two values.","ref":"EAGL.Math.html#lerp/3"},{"type":"type","title":"EAGL.Math.mat2/0","doc":"","ref":"EAGL.Math.html#t:mat2/0"},{"type":"function","title":"EAGL.Math.mat2/4","doc":"Create a 2x2 matrix.","ref":"EAGL.Math.html#mat2/4"},{"type":"function","title":"EAGL.Math.mat2_identity/0","doc":"Create a 2x2 identity matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat2_identity/0"},{"type":"type","title":"EAGL.Math.mat3/0","doc":"","ref":"EAGL.Math.html#t:mat3/0"},{"type":"function","title":"EAGL.Math.mat3/9","doc":"Create a 3x3 matrix.","ref":"EAGL.Math.html#mat3/9"},{"type":"function","title":"EAGL.Math.mat3_identity/0","doc":"Create a 3x3 identity matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat3_identity/0"},{"type":"function","title":"EAGL.Math.mat3_mul/2","doc":"Multiply two 3x3 matrices.\nAll matrices are in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat3_mul/2"},{"type":"function","title":"EAGL.Math.mat3_transpose/1","doc":"Transpose a 3x3 matrix.\nInput and output matrices are in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat3_transpose/1"},{"type":"type","title":"EAGL.Math.mat4/0","doc":"","ref":"EAGL.Math.html#t:mat4/0"},{"type":"function","title":"EAGL.Math.mat4/16","doc":"Create a 4x4 matrix.","ref":"EAGL.Math.html#mat4/16"},{"type":"function","title":"EAGL.Math.mat4_identity/0","doc":"Create a 4x4 identity matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_identity/0"},{"type":"function","title":"EAGL.Math.mat4_inverse/1","doc":"Create the inverse of a 4x4 matrix using the adjugate method.\nReturns the original matrix if it's not invertible (determinant is zero).\nInput and output matrices are in column-major order for OpenGL compatibility.\nNote this is the only function not from the original OpenGl GLM library.","ref":"EAGL.Math.html#mat4_inverse/1"},{"type":"function","title":"EAGL.Math.mat4_look_at/3","doc":"Create a look-at view matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_look_at/3"},{"type":"function","title":"EAGL.Math.mat4_mul/2","doc":"Multiply two 4x4 matrices.\nAll matrices are in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_mul/2"},{"type":"function","title":"EAGL.Math.mat4_ortho/6","doc":"Create an orthographic projection matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_ortho/6"},{"type":"function","title":"EAGL.Math.mat4_perspective/4","doc":"Create a perspective projection matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_perspective/4"},{"type":"function","title":"EAGL.Math.mat4_rotate/2","doc":"Create a rotation matrix from axis and angle.","ref":"EAGL.Math.html#mat4_rotate/2"},{"type":"function","title":"EAGL.Math.mat4_rotate_x/1","doc":"Create a rotation matrix around the X axis.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_rotate_x/1"},{"type":"function","title":"EAGL.Math.mat4_rotate_y/1","doc":"Create a rotation matrix around the Y axis.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_rotate_y/1"},{"type":"function","title":"EAGL.Math.mat4_rotate_z/1","doc":"Create a rotation matrix around the Z axis.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_rotate_z/1"},{"type":"function","title":"EAGL.Math.mat4_scale/1","doc":"Create a scale matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_scale/1"},{"type":"function","title":"EAGL.Math.mat4_transform_point/2","doc":"Multiply a 3D vector by a 4x4 matrix (treating vector as point with w=1).\nReturns a 3D vector with the w component divided out.","ref":"EAGL.Math.html#mat4_transform_point/2"},{"type":"function","title":"EAGL.Math.mat4_transform_vec4/2","doc":"Multiply a 4D vector by a 4x4 matrix.","ref":"EAGL.Math.html#mat4_transform_vec4/2"},{"type":"function","title":"EAGL.Math.mat4_transform_vector/2","doc":"Multiply a 3D vector by a 4x4 matrix (treating vector as direction with w=0).\nUsed for transforming direction vectors (normals, etc.) where translation should be ignored.","ref":"EAGL.Math.html#mat4_transform_vector/2"},{"type":"function","title":"EAGL.Math.mat4_translate/1","doc":"Create a translation matrix.\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_translate/1"},{"type":"function","title":"EAGL.Math.mat4_transpose/1","doc":"Transpose a 4x4 matrix.\nInput and output matrices are in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#mat4_transpose/1"},{"type":"function","title":"EAGL.Math.mix/3","doc":"Mix (linear interpolation) between two values.","ref":"EAGL.Math.html#mix/3"},{"type":"function","title":"EAGL.Math.mod/2","doc":"Modulo operation that works with floats.","ref":"EAGL.Math.html#mod/2"},{"type":"function","title":"EAGL.Math.normalize/1","doc":"Normalize a vector to unit length (2D, 3D, or 4D).","ref":"EAGL.Math.html#normalize/1"},{"type":"function","title":"EAGL.Math.parallel?/3","doc":"Check if two vectors are parallel (dot product close to ±1).","ref":"EAGL.Math.html#parallel?/3"},{"type":"function","title":"EAGL.Math.perpendicular?/3","doc":"Check if two vectors are perpendicular (dot product close to 0).","ref":"EAGL.Math.html#perpendicular?/3"},{"type":"function","title":"EAGL.Math.project/2","doc":"Project vector a onto vector b.","ref":"EAGL.Math.html#project/2"},{"type":"type","title":"EAGL.Math.quat/0","doc":"","ref":"EAGL.Math.html#t:quat/0"},{"type":"function","title":"EAGL.Math.quat/4","doc":"Create a quaternion.\nQuaternions are represented as {x, y, z, w} where w is the scalar component.\nThis follows the (x, y, z, w) convention commonly used in graphics programming.","ref":"EAGL.Math.html#quat/4"},{"type":"function","title":"EAGL.Math.quat_conjugate/1","doc":"Conjugate a quaternion (negate x, y, z components).","ref":"EAGL.Math.html#quat_conjugate/1"},{"type":"function","title":"EAGL.Math.quat_from_axis_angle/2","doc":"Create a quaternion from axis-angle representation.","ref":"EAGL.Math.html#quat_from_axis_angle/2"},{"type":"function","title":"EAGL.Math.quat_from_euler/3","doc":"Create a quaternion from Euler angles (pitch, yaw, roll in radians).","ref":"EAGL.Math.html#quat_from_euler/3"},{"type":"function","title":"EAGL.Math.quat_identity/0","doc":"Create an identity quaternion (no rotation).","ref":"EAGL.Math.html#quat_identity/0"},{"type":"function","title":"EAGL.Math.quat_mul/2","doc":"Multiply two quaternions.","ref":"EAGL.Math.html#quat_mul/2"},{"type":"function","title":"EAGL.Math.quat_normalize/1","doc":"Normalize a quaternion.","ref":"EAGL.Math.html#quat_normalize/1"},{"type":"function","title":"EAGL.Math.quat_rotate_vec3/2","doc":"Rotate a 3D vector by a quaternion.","ref":"EAGL.Math.html#quat_rotate_vec3/2"},{"type":"function","title":"EAGL.Math.quat_slerp/3","doc":"Spherical linear interpolation between two quaternions.","ref":"EAGL.Math.html#quat_slerp/3"},{"type":"function","title":"EAGL.Math.quat_to_mat3/1","doc":"Convert quaternion to rotation matrix (3x3).\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#quat_to_mat3/1"},{"type":"function","title":"EAGL.Math.quat_to_mat4/1","doc":"Convert quaternion to rotation matrix (4x4).\nMatrix is stored in column-major order for OpenGL compatibility.","ref":"EAGL.Math.html#quat_to_mat4/1"},{"type":"function","title":"EAGL.Math.radians/1","doc":"Convert degrees to radians.","ref":"EAGL.Math.html#radians/1"},{"type":"function","title":"EAGL.Math.reflect/2","doc":"Reflect a vector around a normal.","ref":"EAGL.Math.html#reflect/2"},{"type":"function","title":"EAGL.Math.refract/3","doc":"Refract a vector around a normal with given refractive index ratio.","ref":"EAGL.Math.html#refract/3"},{"type":"function","title":"EAGL.Math.reject/2","doc":"Reject vector a from vector b (perpendicular component).","ref":"EAGL.Math.html#reject/2"},{"type":"function","title":"EAGL.Math.sigil_i/2","doc":"A sigil to create index data from a string.\n\nReturns a flat list of integers, perfect for element array buffer indices,\nface definitions, or any other integer-based data.\n\nSupports comments using '#' - everything after '#' on a line is ignored.\nWorks with module attributes since sigils are evaluated at compile time.","ref":"EAGL.Math.html#sigil_i/2"},{"type":"function","title":"Examples - EAGL.Math.sigil_i/2","doc":"# Simple triangle indices\n    ~i\"0 1 2\"\n\n    # Quad indices (two triangles)\n    ~i\"0 1 2  2 3 0\"\n\n    # Cube indices with comments\n    ~i'''\n    # Front face\n    0 1 2  2 3 0\n    # Back face\n    4 5 6  6 7 4\n    # Left face\n    7 3 0  0 4 7\n    # Right face\n    1 5 6  6 2 1\n    # Top face\n    3 2 6  6 7 3\n    # Bottom face\n    0 1 5  5 4 0\n    '''\n\n    # Triangle strip indices\n    ~i'''\n    0 1 2   # first triangle\n    1 3 2   # second triangle (shares edge 1-2)\n    2 3 4   # third triangle (shares edge 2-3)\n    '''","ref":"EAGL.Math.html#sigil_i/2-examples"},{"type":"function","title":"Module Attributes - EAGL.Math.sigil_i/2","doc":"# Define index data as module attributes\n    @quad_indices ~i'''\n    0 1 2  # first triangle\n    2 3 0  # second triangle\n    '''\n\n    @cube_indices ~i'''\n    # All 12 triangles for a cube\n    0 1 2  2 3 0    # front\n    4 5 6  6 7 4    # back\n    7 3 0  0 4 7    # left\n    1 5 6  6 2 1    # right\n    3 2 6  6 7 3    # top\n    0 1 5  5 4 0    # bottom\n    '''\n\n    def get_quad_indices, do: @quad_indices\n    def get_cube_indices, do: @cube_indices","ref":"EAGL.Math.html#sigil_i/2-module-attributes"},{"type":"function","title":"EAGL.Math.sigil_m/2","doc":"A sigil to create a matrix from a string.\n\nAutomatically determines matrix size based on the number of floats:\n- 4 floats: creates a 2x2 matrix\n- 9 floats: creates a 3x3 matrix\n- 16 floats: creates a 4x4 matrix\n\nSupports comments using '#' - everything after '#' on a line is ignored.\nWorks with module attributes since sigils are evaluated at compile time.","ref":"EAGL.Math.html#sigil_m/2"},{"type":"function","title":"Examples - EAGL.Math.sigil_m/2","doc":"# 2x2 matrix (single line)\n    ~m\"1.0 0.0 0.0 1.0\"\n\n    # 2x2 matrix with comments\n    ~m'''\n    1.0 0.0  # row 1\n    0.0 1.0  # row 2\n    '''\n\n    # 3x3 identity matrix with detailed comments\n    ~m'''\n    1.0 0.0 0.0  # X axis\n    0.0 1.0 0.0  # Y axis\n    0.0 0.0 1.0  # Z axis\n    '''\n\n    # 4x4 transformation matrix\n    ~m'''\n    1.0 0.0 0.0 5.0  # X axis + translation\n    0.0 1.0 0.0 0.0  # Y axis\n    0.0 0.0 1.0 0.0  # Z axis\n    0.0 0.0 0.0 1.0  # homogeneous coordinate\n    '''","ref":"EAGL.Math.html#sigil_m/2-examples"},{"type":"function","title":"Module Attributes - EAGL.Math.sigil_m/2","doc":"# Define matrices as module attributes for reuse\n    @identity_4x4 ~m'''\n    1.0 0.0 0.0 0.0  # identity matrix\n    0.0 1.0 0.0 0.0\n    0.0 0.0 1.0 0.0\n    0.0 0.0 0.0 1.0\n    '''\n\n    @projection_matrix ~m'''\n    1.81 0.0  0.0   0.0   # perspective projection\n    0.0  2.41 0.0   0.0   # 45° FOV, 16:9 aspect\n    0.0  0.0 -1.002 -0.2  # near=0.1, far=100.0\n    0.0  0.0 -1.0   0.0\n    '''\n\n    def get_identity, do: @identity_4x4\n    def get_projection, do: @projection_matrix","ref":"EAGL.Math.html#sigil_m/2-module-attributes"},{"type":"function","title":"EAGL.Math.sigil_v/2","doc":"A sigil to create vertex data from a string.\n\nReturns a flat list of floats, perfect for vertex arrays, texture coordinates,\nnormals, colors, or any other vertex attribute data.\n\nSupports comments using '#' - everything after '#' on a line is ignored.\nWorks with module attributes since sigils are evaluated at compile time.","ref":"EAGL.Math.html#sigil_v/2"},{"type":"function","title":"Examples - EAGL.Math.sigil_v/2","doc":"# Simple triangle vertices (x, y, z)\n    ~v\"0.0 0.5 0.0  -0.5 -0.5 0.0  0.5 -0.5 0.0\"\n\n    # Triangle with comments\n    ~v'''\n    0.0  0.5 0.0   # top vertex\n    -0.5 -0.5 0.0  # bottom left\n    0.5  -0.5 0.0  # bottom right\n    '''\n\n    # Quad vertices with texture coordinates\n    ~v'''\n    # Position    # Texture coords\n    -1.0  1.0 0.0  0.0 1.0  # top left\n     1.0  1.0 0.0  1.0 1.0  # top right\n     1.0 -1.0 0.0  1.0 0.0  # bottom right\n    -1.0 -1.0 0.0  0.0 0.0  # bottom left\n    '''\n\n    # Color data (RGB)\n    ~v'''\n    1.0 0.0 0.0  # red\n    0.0 1.0 0.0  # green\n    0.0 0.0 1.0  # blue\n    '''","ref":"EAGL.Math.html#sigil_v/2-examples"},{"type":"function","title":"Module Attributes - EAGL.Math.sigil_v/2","doc":"# Define vertex data as module attributes\n    @triangle_vertices ~v'''\n    0.0  0.5 0.0   # top\n    -0.5 -0.5 0.0  # bottom left\n    0.5  -0.5 0.0  # bottom right\n    '''\n\n    @quad_indices ~v\"0 1 2  2 3 0\"  # triangle indices\n\n    def get_triangle_vertices, do: @triangle_vertices\n    def get_quad_indices, do: @quad_indices","ref":"EAGL.Math.html#sigil_v/2-module-attributes"},{"type":"function","title":"EAGL.Math.sign/1","doc":"Sign function (-1, 0, or 1).","ref":"EAGL.Math.html#sign/1"},{"type":"function","title":"EAGL.Math.smooth_step/3","doc":"Smooth step function (Hermite interpolation).","ref":"EAGL.Math.html#smooth_step/3"},{"type":"function","title":"EAGL.Math.step/2","doc":"Step function (returns 0.0 if x < edge, 1.0 otherwise).","ref":"EAGL.Math.html#step/2"},{"type":"type","title":"EAGL.Math.vec2/0","doc":"","ref":"EAGL.Math.html#t:vec2/0"},{"type":"function","title":"EAGL.Math.vec2/2","doc":"Create a 2D vector.","ref":"EAGL.Math.html#vec2/2"},{"type":"function","title":"EAGL.Math.vec2_zero/0","doc":"Create a zero vector (2D).","ref":"EAGL.Math.html#vec2_zero/0"},{"type":"type","title":"EAGL.Math.vec3/0","doc":"","ref":"EAGL.Math.html#t:vec3/0"},{"type":"function","title":"EAGL.Math.vec3/3","doc":"Create a 3D vector.","ref":"EAGL.Math.html#vec3/3"},{"type":"function","title":"EAGL.Math.vec3_one/0","doc":"Create a 3D vector with all components set to 1.","ref":"EAGL.Math.html#vec3_one/0"},{"type":"function","title":"EAGL.Math.vec3_unit_x/0","doc":"Create the X unit vector (1, 0, 0).","ref":"EAGL.Math.html#vec3_unit_x/0"},{"type":"function","title":"EAGL.Math.vec3_unit_y/0","doc":"Create the Y unit vector (0, 1, 0).","ref":"EAGL.Math.html#vec3_unit_y/0"},{"type":"function","title":"EAGL.Math.vec3_unit_z/0","doc":"Create the Z unit vector (0, 0, 1).","ref":"EAGL.Math.html#vec3_unit_z/0"},{"type":"function","title":"EAGL.Math.vec3_zero/0","doc":"Create a zero vector (3D).","ref":"EAGL.Math.html#vec3_zero/0"},{"type":"type","title":"EAGL.Math.vec4/0","doc":"","ref":"EAGL.Math.html#t:vec4/0"},{"type":"function","title":"EAGL.Math.vec4/4","doc":"Create a 4D vector.","ref":"EAGL.Math.html#vec4/4"},{"type":"function","title":"EAGL.Math.vec4_zero/0","doc":"Create a zero vector (4D).","ref":"EAGL.Math.html#vec4_zero/0"},{"type":"function","title":"EAGL.Math.vec_add/2","doc":"Add two vectors component-wise.","ref":"EAGL.Math.html#vec_add/2"},{"type":"function","title":"EAGL.Math.vec_length/1","doc":"Compute the length (magnitude) of a vector (2D, 3D, or 4D).","ref":"EAGL.Math.html#vec_length/1"},{"type":"function","title":"EAGL.Math.vec_lerp/3","doc":"Linear interpolation between two vectors.","ref":"EAGL.Math.html#vec_lerp/3"},{"type":"function","title":"EAGL.Math.vec_negate/1","doc":"Negate a vector (multiply by -1).","ref":"EAGL.Math.html#vec_negate/1"},{"type":"function","title":"EAGL.Math.vec_scale/2","doc":"Multiply a vector by a scalar.","ref":"EAGL.Math.html#vec_scale/2"},{"type":"function","title":"EAGL.Math.vec_sub/2","doc":"Subtract two vectors component-wise.","ref":"EAGL.Math.html#vec_sub/2"},{"type":"module","title":"EAGL.Shader","doc":"OpenGL shader compilation and program management.\n\nHandles the complex multi-step process of shader compilation, linking,\nand uniform management with Wings3D-inspired error handling.","ref":"EAGL.Shader.html"},{"type":"module","title":"Original Source - EAGL.Shader","doc":"Uniform helper functions (`uloc/2`, `set_uloc/3`) are inspired by Wings3D's\n`wings_gl.erl` module:","ref":"EAGL.Shader.html#module-original-source"},{"type":"module","title":"Usage - EAGL.Shader","doc":"import EAGL.Shader\n    import EAGL.Math\n\n    # Compile and link shaders with typed parameters\n    {:ok, vertex} = create_shader(:vertex, \"vertex.glsl\")\n    {:ok, fragment} = create_shader(:fragment, \"fragment.glsl\")\n    {:ok, program} = create_attach_link([vertex, fragment])\n\n    # Set uniforms with automatic type detection\n    set_uniform(program, \"model_matrix\", model_matrix)\n    set_uniform(program, \"light_position\", vec3(10.0, 10.0, 5.0))\n\n    # Set multiple uniforms efficiently\n    set_uniforms(program, [\n      model: model_matrix,\n      view: view_matrix,\n      projection: projection_matrix\n    ])\n\n    # Use OpenGL directly for program binding\n    :gl.useProgram(program)\n\n    # Clean up when done\n    cleanup_program(program)","ref":"EAGL.Shader.html#module-usage"},{"type":"function","title":"EAGL.Shader.cache_uniform_locations/2","doc":"Convenience function to cache uniform locations for repeated use.\nReturns a map of uniform names to their locations.","ref":"EAGL.Shader.html#cache_uniform_locations/2"},{"type":"function","title":"EAGL.Shader.check_compile_status/1","doc":"Checks if a shader compiled successfully.","ref":"EAGL.Shader.html#check_compile_status/1"},{"type":"function","title":"EAGL.Shader.check_link_status/1","doc":"","ref":"EAGL.Shader.html#check_link_status/1"},{"type":"function","title":"EAGL.Shader.cleanup_program/1","doc":"Cleans up a shader program and all its attached shaders.","ref":"EAGL.Shader.html#cleanup_program/1"},{"type":"function","title":"EAGL.Shader.cleanup_shader/1","doc":"Cleans up a shader.","ref":"EAGL.Shader.html#cleanup_shader/1"},{"type":"function","title":"EAGL.Shader.create_attach_link/1","doc":"Creates a program, attaches shaders, links, and returns the link status.","ref":"EAGL.Shader.html#create_attach_link/1"},{"type":"function","title":"EAGL.Shader.create_shader/2","doc":"Creates and compiles a shader of the specified type with the given source code.\nReturns the shader ID.","ref":"EAGL.Shader.html#create_shader/2"},{"type":"function","title":"EAGL.Shader.get_uniform_location/2","doc":"Get uniform location for a program. Similar to wings_gl:uloc/2.\nReturns the uniform location or -1 if not found.","ref":"EAGL.Shader.html#get_uniform_location/2"},{"type":"type","title":"EAGL.Shader.mat2/0","doc":"","ref":"EAGL.Shader.html#t:mat2/0"},{"type":"type","title":"EAGL.Shader.mat3/0","doc":"","ref":"EAGL.Shader.html#t:mat3/0"},{"type":"type","title":"EAGL.Shader.mat4/0","doc":"","ref":"EAGL.Shader.html#t:mat4/0"},{"type":"type","title":"EAGL.Shader.program_id/0","doc":"","ref":"EAGL.Shader.html#t:program_id/0"},{"type":"type","title":"EAGL.Shader.quat/0","doc":"","ref":"EAGL.Shader.html#t:quat/0"},{"type":"function","title":"EAGL.Shader.set_uniform/3","doc":"Set uniform value with automatic type detection. Similar to wings_gl:set_uloc/3.\nSupports various EAGL.Math types and basic values.","ref":"EAGL.Shader.html#set_uniform/3"},{"type":"function","title":"EAGL.Shader.set_uniform_at_location/2","doc":"Set uniform value at a specific location with automatic type detection.","ref":"EAGL.Shader.html#set_uniform_at_location/2"},{"type":"function","title":"EAGL.Shader.set_uniforms/2","doc":"Convenience function to set multiple uniforms at once.\nTakes a program and a list of {name, value} tuples where name can be atom or string.","ref":"EAGL.Shader.html#set_uniforms/2"},{"type":"type","title":"EAGL.Shader.shader_id/0","doc":"","ref":"EAGL.Shader.html#t:shader_id/0"},{"type":"type","title":"EAGL.Shader.shader_type/0","doc":"","ref":"EAGL.Shader.html#t:shader_type/0"},{"type":"type","title":"EAGL.Shader.uniform_value/0","doc":"","ref":"EAGL.Shader.html#t:uniform_value/0"},{"type":"type","title":"EAGL.Shader.vec2/0","doc":"","ref":"EAGL.Shader.html#t:vec2/0"},{"type":"type","title":"EAGL.Shader.vec3/0","doc":"","ref":"EAGL.Shader.html#t:vec3/0"},{"type":"type","title":"EAGL.Shader.vec4/0","doc":"","ref":"EAGL.Shader.html#t:vec4/0"},{"type":"module","title":"EAGL.Texture","doc":"OpenGL texture management utilities.\n\nHandles texture creation, loading, and parameter configuration with\nWings3D-inspired helper functions, focusing on meaningful abstractions\nrather than thin wrappers around OpenGL calls.","ref":"EAGL.Texture.html"},{"type":"module","title":"Original Source - EAGL.Texture","doc":"Texture management patterns and helper functions are inspired by Wings3D's\n`wings_gl.erl` module:","ref":"EAGL.Texture.html#module-original-source"},{"type":"module","title":"Basic Usage - EAGL.Texture","doc":"import EAGL.Texture\n    import EAGL.Error\n\n    # Load texture from image file (requires optional stb_image dependency)\n    {:ok, texture_id, width, height} = load_texture_from_file(\"priv/images/eagl_logo_black_on_white.jpg\")\n\n    # Or create procedural textures\n    {:ok, texture_id, width, height} = create_checkerboard_texture(256, 32)\n\n    # Manual texture creation and configuration\n    {:ok, texture_id} = create_texture()\n    :gl.bindTexture(@gl_texture_2d, texture_id)\n    set_texture_parameters(wrap_s: :clamp_to_edge, min_filter: :linear)\n    load_texture_data(width, height, pixel_data, format: :rgb)\n    :gl.generateMipmap(@gl_texture_2d)\n    check(\"After generating mipmaps\")\n\n    # Clean up\n    :gl.deleteTextures([texture_id])","ref":"EAGL.Texture.html#module-basic-usage"},{"type":"module","title":"Texture Parameters - EAGL.Texture","doc":"- **Wrapping**: `:repeat`, `:mirrored_repeat`, `:clamp_to_edge`, `:clamp_to_border`\n- **Filtering**: `:nearest`, `:linear`, `:nearest_mipmap_nearest`, `:linear_mipmap_linear`, etc.\n- **Format**: `:rgb`, `:rgba`, `:red`, `:rg`","ref":"EAGL.Texture.html#module-texture-parameters"},{"type":"module","title":"Philosophy - EAGL.Texture","doc":"This module provides substantial helper functions rather than thin wrappers:\n- `create_texture()` - Returns `{:ok, id}` tuples for error handling\n- `set_texture_parameters()` - Converts atoms to GL constants\n- `load_texture_data()` - Handles format/type conversion with defaults\n- `create_checkerboard_texture()` - Generates procedural textures\n\nFor simple operations like binding, unbinding, or generating mipmaps,\ncall the OpenGL functions directly and use `EAGL.Error.check()` as needed.","ref":"EAGL.Texture.html#module-philosophy"},{"type":"function","title":"EAGL.Texture.create_checkerboard_texture/2","doc":"Creates a simple checkerboard texture for testing purposes.\nReturns a tuple of {texture_id, width, height}.","ref":"EAGL.Texture.html#create_checkerboard_texture/2"},{"type":"function","title":"Parameters - EAGL.Texture.create_checkerboard_texture/2","doc":"- `size`: Size of the checkerboard (default: 256)\n- `square_size`: Size of each square (default: 32)","ref":"EAGL.Texture.html#create_checkerboard_texture/2-parameters"},{"type":"function","title":"Examples - EAGL.Texture.create_checkerboard_texture/2","doc":"{:ok, texture_id, width, height} = create_checkerboard_texture()\n    {:ok, texture_id, width, height} = create_checkerboard_texture(128, 16)","ref":"EAGL.Texture.html#create_checkerboard_texture/2-examples"},{"type":"function","title":"EAGL.Texture.create_texture/0","doc":"Creates a new texture object and returns its ID.","ref":"EAGL.Texture.html#create_texture/0"},{"type":"function","title":"EAGL.Texture.create_textures/1","doc":"Creates multiple texture objects and returns their IDs.","ref":"EAGL.Texture.html#create_textures/1"},{"type":"type","title":"EAGL.Texture.filter_mode/0","doc":"","ref":"EAGL.Texture.html#t:filter_mode/0"},{"type":"function","title":"EAGL.Texture.load_texture_data/4","doc":"Loads pixel data into the currently bound texture.","ref":"EAGL.Texture.html#load_texture_data/4"},{"type":"function","title":"Parameters - EAGL.Texture.load_texture_data/4","doc":"- `width`: Texture width in pixels\n- `height`: Texture height in pixels\n- `pixel_data`: Binary containing pixel data\n- `opts`: Options for format and type","ref":"EAGL.Texture.html#load_texture_data/4-parameters"},{"type":"function","title":"Options - EAGL.Texture.load_texture_data/4","doc":"- `internal_format`: Internal storage format (default: :rgb)\n- `format`: Pixel data format (default: :rgb)\n- `type`: Pixel data type (default: :unsigned_byte)\n- `level`: Mipmap level (default: 0)","ref":"EAGL.Texture.html#load_texture_data/4-options"},{"type":"function","title":"Examples - EAGL.Texture.load_texture_data/4","doc":"# RGB data\n    load_texture_data(256, 256, pixel_data, format: :rgb)\n\n    # RGBA data with alpha channel\n    load_texture_data(256, 256, pixel_data,\n      internal_format: :rgba,\n      format: :rgba\n    )","ref":"EAGL.Texture.html#load_texture_data/4-examples"},{"type":"function","title":"EAGL.Texture.load_texture_from_file/2","doc":"Loads a texture from an image file.\n\nRequires the optional `stb_image` dependency. If not available, falls back to\ncheckerboard texture with a helpful warning message.","ref":"EAGL.Texture.html#load_texture_from_file/2"},{"type":"function","title":"Parameters - EAGL.Texture.load_texture_from_file/2","doc":"- `file_path`: Path to the image file (relative to project root or absolute)\n- `opts`: Options for texture loading","ref":"EAGL.Texture.html#load_texture_from_file/2-parameters"},{"type":"function","title":"Options - EAGL.Texture.load_texture_from_file/2","doc":"- `flip_y`: Flip image vertically (default: true, matches OpenGL convention)\n- `fallback_size`: Size of fallback checkerboard if image loading fails (default: 256)\n- `fallback_square_size`: Square size for fallback checkerboard (default: 32)","ref":"EAGL.Texture.html#load_texture_from_file/2-options"},{"type":"function","title":"Examples - EAGL.Texture.load_texture_from_file/2","doc":"# Load EAGL logo\n    {:ok, texture_id, width, height} = load_texture_from_file(\"priv/images/eagl_logo_black_on_white.jpg\")\n\n    # Load with custom options\n    {:ok, texture_id, width, height} = load_texture_from_file(\"container.jpg\", flip_y: false)","ref":"EAGL.Texture.html#load_texture_from_file/2-examples"},{"type":"type","title":"EAGL.Texture.pixel_type/0","doc":"","ref":"EAGL.Texture.html#t:pixel_type/0"},{"type":"function","title":"EAGL.Texture.set_texture_parameters/1","doc":"Sets texture parameters for wrapping and filtering.","ref":"EAGL.Texture.html#set_texture_parameters/1"},{"type":"function","title":"Options - EAGL.Texture.set_texture_parameters/1","doc":"- `wrap_s`: Wrapping mode for S coordinate (default: :repeat)\n- `wrap_t`: Wrapping mode for T coordinate (default: :repeat)\n- `min_filter`: Minification filter (default: :linear)\n- `mag_filter`: Magnification filter (default: :linear)","ref":"EAGL.Texture.html#set_texture_parameters/1-options"},{"type":"function","title":"Examples - EAGL.Texture.set_texture_parameters/1","doc":"# Use default parameters (repeat wrapping, linear filtering)\n    set_texture_parameters()\n\n    # Custom parameters\n    set_texture_parameters(\n      wrap_s: :clamp_to_edge,\n      wrap_t: :clamp_to_edge,\n      min_filter: :nearest,\n      mag_filter: :nearest\n    )","ref":"EAGL.Texture.html#set_texture_parameters/1-examples"},{"type":"type","title":"EAGL.Texture.texture_format/0","doc":"","ref":"EAGL.Texture.html#t:texture_format/0"},{"type":"type","title":"EAGL.Texture.texture_id/0","doc":"","ref":"EAGL.Texture.html#t:texture_id/0"},{"type":"type","title":"EAGL.Texture.wrap_mode/0","doc":"","ref":"EAGL.Texture.html#t:wrap_mode/0"},{"type":"module","title":"EAGL.Window","doc":"OpenGL window management and application lifecycle.\n\nCorrect window creation with cross-platform OpenGL context setup\nand automated event handling based on wings_gl.erl patterns.","ref":"EAGL.Window.html"},{"type":"module","title":"Original Source - EAGL.Window","doc":"Window creation timing, context setup, and event handling patterns are\nbased on Wings3D's `wings_gl.erl` module:","ref":"EAGL.Window.html#module-original-source"},{"type":"module","title":"Usage - EAGL.Window","doc":"defmodule MyApp do\n      use EAGL.Window\n\n      def run_example do\n        # 2D rendering (default)\n        EAGL.Window.run(__MODULE__, \"My 2D App\")\n\n        # 3D rendering with depth testing\n        EAGL.Window.run(__MODULE__, \"My 3D App\", depth_testing: true)\n\n        # Custom size and tutorial mode\n        EAGL.Window.run(__MODULE__, \"Tutorial\",\n          size: {1280, 720},\n          return_to_exit: true\n        )\n      end\n\n      @impl true\n      def setup do\n        # Initialize shaders, load models, set up state\n        {:ok, initial_state}\n      end\n\n      @impl true\n      def render(width, height, state) do\n        # Clear screen and render content\n        :gl.clearColor(0.2, 0.3, 0.3, 1.0)\n        :gl.clear(@gl_color_buffer_bit)\n        # ... render content\n        :ok\n      end\n\n      @impl true\n      def cleanup(state) do\n        # Clean up OpenGL resources\n        :ok\n      end\n    end","ref":"EAGL.Window.html#module-usage"},{"type":"function","title":"EAGL.Window.run/2","doc":"Creates and runs an OpenGL window using the given callback module.\nThe callback module must implement the GLWindowBehaviour.\n\nOptions:\n- size: {width, height} tuple, defaults to {1024, 768}. Sets the initial window size.\n- depth_testing: boolean, defaults to false. When true, enables depth testing and requests a depth buffer.\n- return_to_exit: boolean, defaults to false. When true, pressing ENTER will automatically close the window.","ref":"EAGL.Window.html#run/2"},{"type":"function","title":"EAGL.Window.run/3","doc":"","ref":"EAGL.Window.html#run/3"},{"type":"module","title":"EAGL.Model","doc":"3D model loading and OpenGL integration.\n\nHandles model loading with automatic VAO creation. Currently only supports OBJ files\nbut can be extended to support other file formats.","ref":"EAGL.Model.html"},{"type":"module","title":"Usage - EAGL.Model","doc":"import EAGL.Model\n\n    # Load model directly to VAO (most common)\n    {:ok, model} = load_model_to_vao(\"teapot.obj\")\n\n    # Render the model\n    :gl.bindVertexArray(model.vao)\n    :gl.drawElements(@gl_triangles, model.vertex_count, @gl_unsigned_int, 0)\n\n    # Load model data for custom processing\n    {:ok, model_data} = load_model(\"teapot.obj\")\n\n    # Options for normal generation\n    {:ok, model} = load_model_to_vao(\"teapot.obj\",\n      smooth_normals: true,\n      flip_normal_direction: false\n    )\n\n    # List available models\n    models = list_models()\n\n    # Clean up\n    delete_vao(model.vao)","ref":"EAGL.Model.html#module-usage"},{"type":"function","title":"EAGL.Model.delete_vao/1","doc":"Deletes a vertex array object and its associated buffers.","ref":"EAGL.Model.html#delete_vao/1"},{"type":"function","title":"EAGL.Model.list_models/0","doc":"Lists all available models in the priv/models directory.","ref":"EAGL.Model.html#list_models/0"},{"type":"function","title":"EAGL.Model.load_model/2","doc":"Loads a model from the priv/models directory.\nReturns the processed model data ready for OpenGL.\n\nOptions:\n  - :flip_normal_direction - boolean, set to true to flip normal direction for all models (default: false)\n                             This works for both models with existing normals and models that need generated normals.\n  - :smooth_normals - boolean, set to true to generate smooth normals by averaging across adjacent faces (default: false)\n                      This gives a smoother appearance by eliminating the faceted look.","ref":"EAGL.Model.html#load_model/2"},{"type":"function","title":"EAGL.Model.load_model_to_vao/2","doc":"Loads a model and creates a VAO with the model data.\nReturns {:ok, %{vao: vao, vertex_count: count}} or {:error, reason}\n\nThe VAO will have the following attributes:\n- Location 0: Vertex positions (vec3)\n- Location 1: Texture coordinates (vec2)\n- Location 2: Normals (vec3)\n\nOptions:\n  - :flip_normal_direction - boolean, set to true to flip normal direction for all models (default: false)\n                             This works for both models with existing normals and models that need generated normals.\n                             Useful when model normals are pointing in the wrong direction for your lighting setup.\n  - :smooth_normals - boolean, set to true to generate smooth normals by averaging across adjacent faces (default: false)\n                      This gives a smoother appearance by eliminating the faceted look.\n                      When true, existing normals are ignored and smooth normals are generated.","ref":"EAGL.Model.html#load_model_to_vao/2"},{"type":"module","title":"EAGL.ObjLoader","doc":"Simple Wavefront OBJ file loader.\nCurrently supports vertices, texture coordinates, normals, and faces.","ref":"EAGL.ObjLoader.html"},{"type":"function","title":"EAGL.ObjLoader.load_obj/2","doc":"Loads an OBJ file and returns vertex data ready for OpenGL.\nReturns {:ok, data} where data is a map containing:\n  - :vertices - List of floats in x,y,z order\n  - :tex_coords - List of floats in u,v order\n  - :normals - List of floats in x,y,z order\n  - :indices - List of integers for indexed drawing\n\nOptions:\n  - :flip_normal_direction - boolean, set to true to flip normal direction for all models (default: false)\n                             This works consistently for both models with existing normals and models that need generated normals.\n                             For models with existing normals: negates all normal components\n                             For models without normals: generates normals with flipped direction\n  - :smooth_normals - boolean, set to true to generate smooth normals by averaging across adjacent faces (default: false)\n                      This gives a smoother appearance by eliminating the faceted look.\n                      When true, existing normals are ignored and smooth normals are generated.","ref":"EAGL.ObjLoader.html#load_obj/2"},{"type":"module","title":"EAGL.Const","doc":"Curated set of 300 OpenGL constants for 3D graphics programming. Additional constants\ncan be added as needed from the 4,925 constants in the wx/include/gl.hrl file.\n\nUse this module to inject OpenGL constants as module attributes.","ref":"EAGL.Const.html"},{"type":"module","title":"Usage - EAGL.Const","doc":"defmodule MyModule do\n      use EAGL.Const\n\n      # Now you can use @gl_vertex_shader, @gl_triangles, @gl_texture_2d, etc.\n    end","ref":"EAGL.Const.html#module-usage"},{"type":"module","title":"Categories - EAGL.Const","doc":"This module provides constants for:\n- Shader types and compilation\n- Buffer types and usage patterns\n- Drawing primitives and polygon modes\n- Data types and formats\n- Textures and sampling\n- Blending and transparency\n- Depth and stencil testing\n- Matrices and transformations\n- Vertex attributes and arrays\n- Framebuffers and attachments\n- And much more...","ref":"EAGL.Const.html#module-categories"},{"type":"module","title":"EAGL.Examples.Math","doc":"Comprehensive example demonstrating all EAGL.Math functionality.\nThis showcases vector and quaternion constructor macros, vector operations,\nquaternions, matrices, utility functions, and real-world OpenGL usage patterns.","ref":"EAGL.Examples.Math.html"},{"type":"function","title":"EAGL.Examples.Math.constructor_examples/0","doc":"Example showing various vector and quaternion constructors.","ref":"EAGL.Examples.Math.html#constructor_examples/0"},{"type":"function","title":"EAGL.Examples.Math.lighting_demo/0","doc":"Demonstrate lighting calculations using vectors.","ref":"EAGL.Examples.Math.html#lighting_demo/0"},{"type":"function","title":"EAGL.Examples.Math.matrix_operations_demo/0","doc":"Demonstrate matrix operations.","ref":"EAGL.Examples.Math.html#matrix_operations_demo/0"},{"type":"function","title":"EAGL.Examples.Math.opengl_usage_examples/0","doc":"Example showing how to use these in an OpenGL context.","ref":"EAGL.Examples.Math.html#opengl_usage_examples/0"},{"type":"function","title":"EAGL.Examples.Math.quaternion_operations_demo/0","doc":"Demonstrate quaternion operations.","ref":"EAGL.Examples.Math.html#quaternion_operations_demo/0"},{"type":"function","title":"EAGL.Examples.Math.run_example/0","doc":"Run the math example - consistent interface with other examples.","ref":"EAGL.Examples.Math.html#run_example/0"},{"type":"function","title":"EAGL.Examples.Math.transformation_pipeline_demo/0","doc":"Demonstrate typical OpenGL transformation pipeline.","ref":"EAGL.Examples.Math.html#transformation_pipeline_demo/0"},{"type":"function","title":"EAGL.Examples.Math.utility_functions_demo/0","doc":"Demonstrate utility and trigonometric functions.","ref":"EAGL.Examples.Math.html#utility_functions_demo/0"},{"type":"function","title":"EAGL.Examples.Math.vector_operations_demo/0","doc":"Demonstrate vector operations.","ref":"EAGL.Examples.Math.html#vector_operations_demo/0"},{"type":"module","title":"EAGL.Examples.Teapot","doc":"Draw a 3D teapot with Phong shading.\nPress ENTER to quit","ref":"EAGL.Examples.Teapot.html"},{"type":"function","title":"EAGL.Examples.Teapot.run_example/0","doc":"","ref":"EAGL.Examples.Teapot.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"LearnOpenGL 2.1 - Hello Triangle\n\nThis example demonstrates your first triangle in OpenGL - the foundation of all 3D graphics.\nIt corresponds to the Hello Triangle tutorial in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"In the original LearnOpenGL C++ tutorial, this example introduces the core OpenGL rendering pipeline:\nvertex data → vertex shader → primitive assembly → fragment shader → framebuffer.\n\nEAGL's framework simplifies the setup while preserving all educational concepts:\n- Shader compilation and linking are handled by EAGL.Shader helpers\n- VAO/VBO creation uses EAGL.Buffer convenience functions\n- Error checking and resource cleanup are automated\n- The core OpenGL concepts remain unchanged and visible","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"1. **Vertex Data**: Defining triangle vertices in normalized device coordinates (-1 to 1)\n2. **Vertex Buffer Objects (VBO)**: Storing vertex data in GPU memory\n3. **Vertex Array Objects (VAO)**: Configuring how vertex data is interpreted\n4. **Vertex Shaders**: Processing each vertex (position transformation)\n5. **Fragment Shaders**: Determining pixel colors\n6. **Rendering Pipeline**: glDrawArrays() triggers the complete pipeline","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-original-tutorial-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"- Understanding normalized device coordinates (NDC)\n- The relationship between VBOs and VAOs\n- How shaders process vertices and fragments\n- The OpenGL rendering pipeline flow\n- Basic primitive rendering with glDrawArrays","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-key-learning-points"},{"type":"module","title":"Triangle Geometry - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"The triangle is defined by 3 vertices in normalized device coordinates:\n```\n    (0.0, 0.5)\n       /\\\n      /  \\\n     /    \\\n    /______\\\n(-0.5,-0.5) (0.5,-0.5)\n```","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-triangle-geometry"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"- **1.1 Hello Window**: Just a black window (no geometry)\n- **1.2 Hello Window Clear**: Custom clear color (no geometry)\n- **2.1 Hello Triangle**: First actual geometry rendering with shaders","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangle.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"LearnOpenGL 2.3 - Hello Triangle Exercise 1 (Two Triangles Side by Side)\n\nThis example demonstrates drawing multiple triangles with a single draw call.\nIt solves the first exercise from the Hello Triangle tutorial.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"In the original LearnOpenGL C++ tutorial, this exercise asks you to draw 2 triangles\nnext to each other using glDrawArrays by adding more vertices to your data.\n\nEAGL's framework maintains the same approach:\n- Single VAO/VBO containing all vertex data\n- One glDrawArrays call renders both triangles\n- No indexing - each triangle uses 3 unique vertices","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Exercise - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"**Exercise 1**: Try to draw 2 triangles next to each other using glDrawArrays\nby adding more vertices to your data.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-original-tutorial-exercise"},{"type":"module","title":"Solution Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"1. **Multiple Primitives**: Drawing 2 triangles with one draw call\n2. **Vertex Array Layout**: 6 vertices arranged as 2 separate triangles\n3. **Spatial Positioning**: Placing triangles side by side in NDC space\n4. **Draw Call Efficiency**: Single glDrawArrays for multiple primitives\n5. **Vertex Ordering**: Understanding how vertices form triangles","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-solution-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"- How to structure vertex data for multiple primitives\n- The relationship between vertex count and triangle count\n- Positioning geometry in normalized device coordinates\n- When to use glDrawArrays vs glDrawElements\n- Understanding primitive assembly from vertex streams","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-key-learning-points"},{"type":"module","title":"Triangle Geometry - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"Two triangles positioned side by side:\n```\nLeft Triangle:     Right Triangle:\n    /\\                 /\\\n   /  \\               /  \\\n  /____\\             /____\\\n```\n\n6 vertices total: 3 for left triangle + 3 for right triangle\nNo vertex sharing (unlike indexed rendering)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-triangle-geometry"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"- **2.1 Hello Triangle**: 1 triangle, 3 vertices\n- **2.2 Hello Triangle Indexed**: 1 rectangle (2 triangles), 4 vertices + indices\n- **2.3 Exercise 1**: 2 triangles, 6 vertices (no sharing)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise1.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"LearnOpenGL 2.4 - Hello Triangle Exercise 2 (Rectangle with EBO)\n\nThis example demonstrates creating two triangles using Element Buffer Objects.\nIt solves the second exercise from the Hello Triangle tutorial.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"In the original LearnOpenGL C++ tutorial, this exercise asks you to create two triangles\nusing a single VAO and VBO, with an Element Buffer Object (EBO) to avoid vertex duplication.\n\nEAGL's framework maintains the same approach:\n- Single VAO/VBO/EBO for efficient vertex sharing\n- glDrawElements for indexed rendering\n- 4 vertices create 2 triangles through clever indexing","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Exercise - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"**Exercise 2**: Now create the same 2 triangles using an EBO so there are only 4 vertices\nin total instead of 6. This is a solution to the previous exercise using indices.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-original-tutorial-exercise"},{"type":"module","title":"Solution Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"1. **Element Buffer Objects (EBO)**: Storing indices for vertex reuse\n2. **Vertex Sharing**: 4 vertices used to create 2 triangles\n3. **Indexed Rendering**: glDrawElements with index buffer\n4. **Memory Efficiency**: Reducing vertex data through sharing\n5. **Rectangle Formation**: Two triangles forming a quad","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-solution-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"- How EBOs enable vertex sharing between primitives\n- The efficiency difference between indexed and non-indexed rendering\n- Understanding triangle winding order and connectivity\n- When indexed rendering provides the most benefit\n- The relationship between vertex count and primitive count","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-key-learning-points"},{"type":"module","title":"Rectangle Geometry - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"Two triangles sharing vertices to form a rectangle:\n```\n3 ---- 2\n|    / |\n|   /  |\n|  /   |\n| /    |\n0 ---- 1\n```\n\n4 vertices total, 6 indices: [0,1,3, 1,2,3]\nTriangle 1: vertices 0,1,3 | Triangle 2: vertices 1,2,3","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-rectangle-geometry"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"- **2.3 Exercise 1**: 2 triangles, 6 vertices (no sharing)\n- **2.4 Exercise 2**: 2 triangles, 4 vertices + indices (vertex sharing)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise2.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"LearnOpenGL 2.5 - Hello Triangle Exercise 3 (Multiple Shader Programs)\n\nThis example demonstrates using multiple shader programs to render different colored triangles.\nIt solves the third exercise from the Hello Triangle tutorial.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"In the original LearnOpenGL C++ tutorial, this exercise asks you to create two triangles\nthat render with different colors by using different fragment shaders.\n\nEAGL's framework maintains the same approach:\n- Two separate VAO/VBO pairs for independent geometry\n- Shared vertex shader between both programs\n- Different fragment shaders for different colors\n- Multiple draw calls with different shader programs","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Exercise - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"**Exercise 3**: Create two shader programs where the second program uses a different\nfragment shader that outputs the color yellow; draw both triangles again where one\noutputs the color orange and the other outputs the color yellow.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-original-tutorial-exercise"},{"type":"module","title":"Solution Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"1. **Multiple Shader Programs**: Creating and managing separate programs\n2. **Shader Reuse**: Same vertex shader used in both programs\n3. **Fragment Shader Variants**: Different colors from different shaders\n4. **Separate Geometry**: Independent VAO/VBO pairs\n5. **Multi-Pass Rendering**: Multiple draw calls with different programs","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-solution-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"- How to create and manage multiple shader programs\n- Sharing shaders between different programs\n- The relationship between shader programs and rendering state\n- When to use separate geometry vs shared geometry\n- Understanding the cost of shader program switches","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-key-learning-points"},{"type":"module","title":"Triangle Geometry and Colors - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"Two separate triangles with different colors:\n```\nLeft Triangle (Orange)    Right Triangle (Yellow)\n      /\\                       /\\\n     /  \\                     /  \\\n    /____\\                   /____\\\n```\n\n- Left triangle: Orange fragment shader\n- Right triangle: Yellow fragment shader\n- Same vertex shader for both triangles","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-triangle-geometry-and-colors"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"- **2.3 Exercise 1**: 2 triangles, 1 shader program, same color\n- **2.4 Exercise 2**: 2 triangles (rectangle), 1 shader program, same color\n- **2.5 Exercise 3**: 2 triangles, 2 shader programs, different colors","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleExercise3.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"LearnOpenGL 2.2 - Hello Triangle Indexed (Element Buffer Objects)\n\nThis example demonstrates Element Buffer Objects (EBO) for efficient indexed rendering.\nIt shows how to draw a rectangle using two triangles with shared vertices.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"In the original LearnOpenGL C++ tutorial, this example introduces Element Buffer Objects (EBOs)\nas a way to avoid vertex duplication when drawing complex shapes.\n\nEAGL's framework maintains the same concepts while providing helper functions:\n- EBO creation and binding are handled by EAGL.Buffer functions\n- The core concept of indexed rendering remains unchanged\n- glDrawElements vs glDrawArrays distinction is preserved","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"1. **Element Buffer Objects (EBO)**: Storing vertex indices for reuse\n2. **Indexed Rendering**: Drawing with glDrawElements instead of glDrawArrays\n3. **Vertex Sharing**: Using 4 vertices to draw 2 triangles (6 vertices worth)\n4. **Memory Efficiency**: Avoiding duplicate vertex data\n5. **Triangle Assembly**: How indices define triangle connectivity","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-original-tutorial-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"- Understanding indexed vs non-indexed rendering\n- Memory efficiency through vertex sharing\n- The relationship between vertices and indices\n- When to use EBOs vs simple vertex arrays\n- Triangle winding order and face culling implications","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-key-learning-points"},{"type":"module","title":"Rectangle Geometry - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"The rectangle is defined by 4 vertices with 6 indices (2 triangles):\n```\n3 ---- 2\n|    / |\n|   /  |\n|  /   |\n| /    |\n0 ---- 1\n```\n\nVertices: 4 unique positions\nIndices: [0,1,3, 1,2,3] (2 triangles sharing vertices 0, 1, and 3)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-rectangle-geometry"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"- **2.1 Hello Triangle**: 3 vertices, glDrawArrays (no sharing)\n- **2.2 Hello Triangle Indexed**: 4 vertices + indices, glDrawElements (vertex sharing)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloTriangleIndexed.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow","doc":"LearnOpenGL 1.1 - Hello Window\n\nThis example demonstrates basic window creation and OpenGL context setup.\nIt corresponds to the first rendering example in the LearnOpenGL tutorial series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow","doc":"In the original LearnOpenGL C++ tutorial, this example shows a completely black window\nbecause no clearing is performed in the render loop. However, EAGL's Window framework\nautomatically handles basic OpenGL setup including an initial clear for clean state.\n\nTo maintain the pedagogical value while working within EAGL's design:\n- We set the clear color to black (0.0, 0.0, 0.0) to match the original example\n- The framework handles the actual clearing, but we demonstrate the concept\n- Comments explain what would happen without framework assistance","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow","doc":"1. **Window Creation**: EAGL.Window handles GLFW setup and OpenGL context creation\n2. **Render Loop**: The framework manages the main loop and buffer swapping\n3. **Basic Rendering**: Shows the foundation for all subsequent examples\n4. **Input Handling**: ENTER key to exit (handled by framework)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html#module-original-tutorial-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow","doc":"- Understanding the render loop concept\n- OpenGL context and window management\n- The importance of clearing buffers\n- Foundation for more complex rendering","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html#module-key-learning-points"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindow.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear","doc":"LearnOpenGL 1.2 - Hello Window Clear\n\nThis example demonstrates setting a custom clear color for the OpenGL framebuffer.\nIt builds upon the 1.1 Hello Window example by showing how to control the background color.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear","doc":"In the original LearnOpenGL C++ tutorial, this example introduces the concept of:\n- `glClearColor()` - setting the clear color state\n- `glClear(GL_COLOR_BUFFER_BIT)` - actually clearing the buffer\n\nEAGL's Window framework automatically handles clearing for clean state, but this example\ndemonstrates the pedagogical concepts by:\n- Explicitly calling `glClearColor()` to set a custom color\n- Showing the difference between state-setting and state-using functions\n- Explaining the OpenGL state machine concept","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html#module-framework-adaptation-notes"},{"type":"module","title":"Original Tutorial Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear","doc":"1. **OpenGL State Machine**: `glClearColor` sets state, `glClear` uses that state\n2. **Color Buffer Clearing**: Essential for preventing visual artifacts\n3. **RGBA Color Values**: Understanding the 0.0-1.0 range for color components\n4. **Render Loop Integration**: Where clearing fits in the rendering pipeline","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html#module-original-tutorial-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear","doc":"- Understanding OpenGL's state-setting vs state-using functions\n- The importance of clearing buffers each frame\n- How color values work in OpenGL (0.0-1.0 range)\n- The difference between this and 1.1 (black vs colored background)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html#module-key-learning-points"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.HelloWindowClear.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"LearnOpenGL 3.3 - Shaders Class\n\nThis example demonstrates the same vertex colour interpolation as 3.2, but emphasises\nthe importance of shader abstraction and organisation. It shows how to structure\nshader management in a clean, reusable way.\nIt corresponds to the Shaders Class tutorial in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"In the original LearnOpenGL C++ tutorial, this example introduces shader class abstraction:\n- How to encapsulate shader compilation, linking, and error handling\n- How to create reusable shader management utilities\n- How to provide clean APIs for setting uniforms and using shaders\n- The importance of proper resource management and cleanup\n\nEAGL's framework already provides this abstraction through EAGL.Shader:\n- create_shader/2 handles compilation with automatic error reporting\n- create_attach_link/1 handles linking with error checking\n- set_uniform/3 provides type-safe uniform setting\n- All following the same patterns as the original tutorial's Shader class","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#module-framework-adaptation-notes"},{"type":"module","title":"Key Concept: Shader Abstraction - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"**Why shader classes/modules are important:**\n- Encapsulate complex OpenGL shader operations\n- Provide error handling and debugging information\n- Enable code reuse across multiple examples\n- Simplify shader management in larger applications\n- Abstract away low-level OpenGL details\n\n**EAGL.Shader Module Features:**\n- Automatic shader compilation with error reporting\n- File-based shader loading from priv/shaders/\n- Type-safe uniform setting with automatic type detection\n- Resource cleanup and management\n- Consistent error handling patterns","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#module-key-concept-shader-abstraction"},{"type":"module","title":"Visual Result - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"Identical to 3.2 Shaders Interpolation:\n- Bottom-right corner: Red (1.0, 0.0, 0.0)\n- Bottom-left corner: Green (0.0, 1.0, 0.0)\n- Top center: Blue (0.0, 0.0, 1.0)\n- Interior pixels: Smooth interpolation between these colors","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#module-visual-result"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"- **3.2 Shaders Interpolation**: Focus on interpolation concept\n- **3.3 Shaders Class**: Same visual result, emphasis on clean shader abstraction","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersClass.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"LearnOpenGL 3.4 - Shaders Exercise 1\n\nThis example demonstrates how to modify vertex positions in the vertex shader\nto create an upside-down triangle. It shows how vertex transformations can be\napplied directly in the shader rather than modifying vertex data.\nIt corresponds to the first shader exercise in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-original-c-source"},{"type":"module","title":"Exercise Description - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"**Original Exercise:** \"Adjust the vertex shader so that the triangle is upside down\"\n\n**Solution Approach:** Negate the y-coordinate in the vertex shader:\n- Original: `gl_Position = vec4(aPos, 1.0);`\n- Modified: `gl_Position = vec4(aPos.x, -aPos.y, aPos.z, 1.0);`","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-exercise-description"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"In the original LearnOpenGL C++ tutorial, this exercise teaches:\n- How vertex shaders can transform vertex positions\n- The difference between modifying data vs. modifying shaders\n- Understanding coordinate systems and transformations\n- How simple mathematical operations in shaders affect rendering\n\nEAGL's framework preserves all these concepts:\n- Same vertex data as previous examples (no changes needed)\n- Transformation happens entirely in the vertex shader\n- Demonstrates shader-based coordinate manipulation","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-framework-adaptation-notes"},{"type":"module","title":"Key Concept: Vertex Transformation - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"**Why transform in the vertex shader:**\n- More efficient than modifying vertex data on CPU\n- Can be combined with other transformations (matrices, etc.)\n- Demonstrates the power and flexibility of programmable shaders\n- Shows how shaders can manipulate geometry dynamically\n\n**Coordinate System Understanding:**\n- OpenGL uses right-handed coordinate system\n- Y-axis points up in normalized device coordinates\n- Negating Y flips the triangle vertically\n- This is a fundamental transformation concept","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-key-concept-vertex-transformation"},{"type":"module","title":"Visual Result - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"Triangle with interpolated colors but flipped upside-down:\n- Top corners: Red (right) and Green (left) - now at bottom\n- Bottom center: Blue - now at top\n    - Same colour interpolation as 3.2/3.3, just flipped","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-visual-result"},{"type":"module","title":"Learning Objectives - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"- Understanding vertex shader transformations\n- Coordinate system manipulation\n- Difference between data modification vs. shader modification\n- Basic mathematical operations in GLSL","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-learning-objectives"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise1.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"LearnOpenGL 3.5 - Shaders Exercise 2\n\nThis example demonstrates using a uniform variable to apply a horizontal offset\nto the triangle position. It combines uniform variables with vertex transformations\nto show how application data can control shader behavior.\nIt corresponds to the second shader exercise in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-original-c-source"},{"type":"module","title":"Exercise Description - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"**Original Exercise:** \"Specify a horizontal offset via a uniform and move the triangle\nto the right side of the screen in the vertex shader using this offset value\"\n\n**Solution Approach:** Add uniform to vertex shader and apply to x-coordinate:\n- Add: `uniform float xOffset;` to vertex shader\n- Modify: `gl_Position = vec4(aPos.x + xOffset, aPos.y, aPos.z, 1.0);`\n- Set uniform from application: `set_uniform(program, \"xOffset\", 0.5)`","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-exercise-description"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"In the original LearnOpenGL C++ tutorial, this exercise teaches:\n- How to combine uniforms with vertex transformations\n- Passing application data to control shader behavior\n- Understanding the difference between vertex attributes and uniforms\n- How uniforms affect all vertices in a draw call uniformly\n\nEAGL's framework preserves all these concepts:\n- EAGL.Shader.set_uniform/3 handles the uniform setting with type detection\n- Same vertex data, transformation applied via uniform in shader\n- Demonstrates application-controlled geometry manipulation","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-framework-adaptation-notes"},{"type":"module","title":"Key Concept: Uniform-Controlled Transformations - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"**Why use uniforms for transformations:**\n- Can change transformation without recompiling shaders\n- Same transformation applied to all vertices in draw call\n- Efficient way to animate or control geometry\n- Foundation for more complex transformations (matrices)\n\n**Uniform vs. Vertex Attribute:**\n- Uniform: Same value for all vertices (global to draw call)\n- Vertex Attribute: Different value per vertex (per-vertex data)\n- Uniforms perfect for transformations, lighting parameters, time, etc.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-key-concept-uniform-controlled-transformations"},{"type":"module","title":"Visual Result - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"Triangle with interpolated colors moved to the right side of screen:\n    - Same colour interpolation as 3.2/3.3\n- Horizontally offset by uniform value (0.5 units to the right)\n- Demonstrates uniform-controlled positioning","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-visual-result"},{"type":"module","title":"Learning Objectives - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"- Understanding uniform variables in vertex shaders\n- Combining uniforms with vertex transformations\n- Application-controlled shader behavior\n- Difference between uniforms and vertex attributes","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-learning-objectives"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise2.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"LearnOpenGL 3.6 - Shaders Exercise 3\n\nThis example demonstrates outputting vertex position as fragment color, showing\nhow vertex shader outputs are interpolated across the triangle surface and how\ncoordinate values can be visualized as colors.\nIt corresponds to the third shader exercise in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-original-c-source"},{"type":"module","title":"Exercise Description - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"**Original Exercise:** \"Output the vertex position to the fragment shader using the\n`out` keyword and set the fragment's color equal to this vertex position (see how\neven the vertex position values are interpolated across the triangle). Once you\nmanaged to do this; try to answer the following question: why is the bottom-left\nside of our triangle black?\"\n\n**Solution Approach:**\n- Add `out vec3 vertexPos;` to vertex shader\n- Set `vertexPos = aPos;` in vertex shader\n- Add `in vec3 vertexPos;` to fragment shader\n- Set `FragColor = vec4(vertexPos, 1.0);` in fragment shader\n\n**Answer to Question:** The bottom-left is black because:\n- Bottom-left vertex position is (-0.5, -0.5, 0.0)\n- Negative values in RGB color space are clamped to 0.0 (black)\n- Only positive position values create visible colors","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-exercise-description"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"In the original LearnOpenGL C++ tutorial, this exercise teaches:\n- How vertex shader outputs are interpolated by the rasterizer\n- Understanding the relationship between coordinate spaces and color spaces\n- Visualizing geometric data as color information\n- How negative values affect color rendering\n\nEAGL's framework preserves all these concepts:\n- Same vertex data and shader input/output patterns\n- Demonstrates interpolation and coordinate-to-color mapping\n- Shows the visual effects of coordinate system properties","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-framework-adaptation-notes"},{"type":"module","title":"Key Concept: Position-to-Color Mapping - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"**Why this visualization is educational:**\n- Makes interpolation visible and understandable\n- Shows how coordinate values map to color values\n- Demonstrates the effect of negative coordinates\n- Helps understand the rasterization process\n\n**Coordinate to Color Mapping:**\n- X coordinate → Red channel\n- Y coordinate → Green channel\n- Z coordinate → Blue channel\n- Negative values clamped to 0.0 (black)\n- Positive values create visible colors","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-key-concept-position-to-color-mapping"},{"type":"module","title":"Visual Result and Analysis - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"Triangle with position-based coloring:\n- **Bottom-right (+0.5, -0.5, 0.0)**: Red channel 0.5, others 0.0 → Dark red\n- **Bottom-left (-0.5, -0.5, 0.0)**: All negative/zero → Black\n- **Top center (0.0, +0.5, 0.0)**: Green channel 0.5, others 0.0 → Dark green\n- **Interior pixels**: Interpolated between these values\n\n**Why bottom-left is black:**\n- Position (-0.5, -0.5, 0.0) has negative X and Y\n- RGB color channels can't be negative (clamped to 0.0)\n- Result: (0.0, 0.0, 0.0) = black","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-visual-result-and-analysis"},{"type":"module","title":"Learning Objectives - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"- Understanding vertex shader output interpolation\n- Coordinate space to color space mapping\n- Effects of negative values in color calculations\n- Visualizing geometric data through color","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-learning-objectives"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersExercise3.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"LearnOpenGL 3.2 - Shaders Interpolation\n\nThis example demonstrates vertex colour interpolation in shaders - how colours assigned to vertices\nare automatically interpolated across the triangle surface by the rasterizer.\nIt corresponds to the Shaders Interpolation tutorial in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"In the original LearnOpenGL C++ tutorial, this example introduces vertex attributes for colours:\n- How to pass colour data as vertex attributes alongside position data\n- How to declare multiple vertex attributes in shaders\n- How the rasterizer automatically interpolates values between vertices\n- Understanding the difference between per-vertex and per-fragment data\n\nEAGL's framework preserves all these concepts while providing convenience functions:\n- EAGL.Buffer.create_vertex_array() handles the VAO/VBO setup with multiple attributes\n- The core OpenGL vertex attribute concepts remain unchanged and visible","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#module-framework-adaptation-notes"},{"type":"module","title":"Key Concept: Fragment Interpolation - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"**What happens during rasterization:**\n- Triangle has 3 vertices with different colours (red, green, blue)\n- Rasterizer determines which pixels are inside the triangle\n- For each pixel, it calculates interpolated values based on distance from vertices\n- A pixel 70% of the way from red to blue vertex gets 30% red + 70% blue\n- This creates smooth colour gradients across the triangle surface\n\n**Original Tutorial Concepts Demonstrated:**\n\n1. **Multiple Vertex Attributes**: Position (location 0) and Colour (location 1)\n2. **Vertex Attribute Stride**: 6 floats per vertex (3 position + 3 colour)\n3. **Vertex Attribute Offsets**: Position at offset 0, Colour at offset 3\n4. **Shader Input/Output**: Vertex shader outputs colour, fragment shader receives interpolated colour\n5. **Automatic Interpolation**: GPU automatically interpolates all vertex shader outputs","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#module-key-concept-fragment-interpolation"},{"type":"module","title":"Visual Result - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"The triangle displays a colour gradient:\n- Bottom-right corner: Red (1.0, 0.0, 0.0)\n- Bottom-left corner: Green (0.0, 1.0, 0.0)\n- Top center: Blue (0.0, 0.0, 1.0)\n    - Interior pixels: Smooth interpolation between these colours","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#module-visual-result"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"- **3.1 Shaders Uniform**: Single colour for entire triangle, changed via uniform\n- **3.2 Shaders Interpolation**: Different colour per vertex, interpolated across surface","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersInterpolation.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"LearnOpenGL 3.1 - Shaders Uniform\n\nThis example demonstrates uniform variables in shaders - global variables that remain constant\nacross all vertices in a draw call but can be updated between draw calls.\nIt corresponds to the Shaders Uniform tutorial in the LearnOpenGL series.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"In the original LearnOpenGL C++ tutorial, this example introduces uniform variables:\n- How to declare uniforms in GLSL\n- How to get uniform locations from the application\n- How to update uniform values using glUniform* functions\n- Creating animated effects by updating uniforms per frame\n\nEAGL's framework preserves all these concepts while providing convenience functions:\n- EAGL.Shader.set_uniform() handles the glGetUniformLocation + glUniform* pattern\n- Error checking and type detection are automated\n- The core OpenGL uniform concepts remain unchanged and visible","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-framework-adaptation-notes"},{"type":"module","title":"Key Difference: Animation Timing - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"**Original LearnOpenGL approach:** Time calculation happens directly in the render loop:\n```c++\nwhile (!glfwWindowShouldClose(window)) {\n    float timeValue = glfwGetTime();  // Get time in render loop\n    float greenValue = sin(timeValue) / 2.0f + 0.5f;\n    glUniform4f(vertexColorLocation, 0.0f, greenValue, 0.0f, 1.0f);\n    // ... render\n}\n```\n\n**EAGL approach:** Uses the tick handler pattern for clean separation of concerns:\n- `handle_event(:tick, state)` updates time state each frame\n- `render/3` uses the current time state for animation\n- This separates state management from rendering logic","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-key-difference-animation-timing"},{"type":"module","title":"Original Tutorial Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"1. **Uniform Declaration**: Declaring `uniform vec4 ourColor` in the fragment shader\n2. **Uniform Location**: Getting the uniform location using `glGetUniformLocation`\n3. **Uniform Updates**: Using `glUniform4f` to update the uniform value\n4. **Animation**: Updating the uniform each frame using `sin(time)` for color cycling\n5. **Shader Communication**: Passing data from application to shader via uniforms","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-original-tutorial-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"- Understanding the difference between attributes and uniforms\n- Uniforms are global variables that stay constant per draw call\n- Uniforms can be updated between draw calls to create dynamic effects\n- Time-based animation using mathematical functions\n- The OpenGL uniform system and how it bridges CPU and GPU","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-key-learning-points"},{"type":"module","title":"Color Animation - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"The triangle color cycles through a green color animation:\n- Color varies from dark green (0.0) to bright green (1.0)\n- Uses `sin(time)` to create smooth oscillation\n- Red and blue components remain at 0.0\n- Alpha remains at 1.0 (fully opaque)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-color-animation"},{"type":"module","title":"Difference from Previous Examples - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"- **2.1 Hello Triangle**: Static orange triangle with hardcoded fragment color\n- **3.1 Shaders Uniform**: Dynamic color animation using uniform variables","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-difference-from-previous-examples"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.ShadersUniform.html#run_example/0"},{"type":"module","title":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"LearnOpenGL 4.1 - Textures\n\nThis example demonstrates basic texture mapping in OpenGL - applying 2D images to 3D geometry.\nIt corresponds to the Textures tutorial in the LearnOpenGL series and showcases EAGL's\ncomprehensive texture loading capabilities.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html"},{"type":"module","title":"Original C++ Source - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"This example is based on the original LearnOpenGL C++ tutorial:","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-original-c-source"},{"type":"module","title":"Framework Adaptation Notes - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"In the original LearnOpenGL C++ tutorial, this example introduces textures:\n- Loading image files using stb_image.h library\n- Creating OpenGL texture objects\n- Setting texture parameters (wrapping, filtering)\n- Texture coordinates and sampling\n- Binding textures before rendering\n\nEAGL's framework preserves all these concepts while providing enhanced functionality:\n- **Real Image Loading**: Uses the same stb_image library as the original tutorial\n- **Graceful Degradation**: Automatic fallback to checkerboard patterns when images unavailable\n- **Pixel Alignment Handling**: Proper handling of non-4-byte-aligned image widths\n- **Y-Axis Correction**: Automatic Y-flip to match OpenGL coordinate conventions\n- **Format Detection**: Automatic channel detection (RGB, RGBA) and format selection","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-framework-adaptation-notes"},{"type":"module","title":"EAGL vs Original Implementation - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"**Original LearnOpenGL approach:** Uses stb_image.h to load container.jpg:\n```c++\nint width, height, nrChannels;\nunsigned char *data = stbi_load(\"container.jpg\", &width, &height, &nrChannels, 0);\nglTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);\nstbi_image_free(data);\n```\n\n**EAGL approach:** Enhanced image loading with comprehensive error handling:\n```elixir\n{:ok, texture_id, width, height} = load_texture_from_file(\"priv/images/eagl_logo_black_on_white.png\")\n```\n- **Same underlying library**: Uses stb_image (optional dependency)\n- **Enhanced error handling**: `{:ok, result}` tuples and comprehensive fallbacks\n- **Pixel alignment fix**: `glPixelStorei(GL_UNPACK_ALIGNMENT, 1)` for arbitrary image widths\n- **Coordinate system handling**: Automatic Y-flip for OpenGL convention\n- **Educational value**: Works with or without external dependencies","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-eagl-vs-original-implementation"},{"type":"module","title":"Technical Improvements - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"This implementation includes several technical enhancements over basic texture loading:\n\n1. **Pixel Alignment**: Handles images whose width × channels isn't 4-byte aligned\n   - The 418×418 EAGL logo tests this fix (418 × 4 = 1672 bytes per row)\n   - Prevents diagonal texture skewing common with arbitrary image dimensions\n\n2. **Y-Axis Handling**: Proper coordinate system conversion\n   - Images typically have (0,0) at top-left\n   - OpenGL expects (0,0) at bottom-left\n   - Automatic Y-flip ensures textures appear right-side up\n\n3. **Format Detection**: Automatic channel count handling\n   - 1 channel → GL_RED\n   - 2 channels → GL_RG\n   - 3 channels → GL_RGB\n   - 4 channels → GL_RGBA","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-technical-improvements"},{"type":"module","title":"Original Tutorial Concepts Demonstrated - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"1. **Texture Objects**: Creating texture IDs with `create_texture()`\n2. **Texture Binding**: Binding textures with `:gl.bindTexture()`\n3. **Texture Parameters**: Setting wrap and filter modes with `set_texture_parameters()`\n4. **Texture Data**: Loading pixel data with `load_texture_data()`\n5. **Mipmaps**: Generating mipmaps with `:gl.generateMipmap()`\n6. **Texture Coordinates**: Adding texture coordinates to vertex data\n7. **Texture Sampling**: Using `sampler2D` and `texture()` in fragment shader","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-original-tutorial-concepts-demonstrated"},{"type":"module","title":"Key Learning Points - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"- Understanding texture coordinates (0,0 to 1,1 mapping)\n- Texture wrapping modes (repeat, clamp, etc.)\n- Texture filtering (nearest vs linear)\n- The relationship between vertex attributes and fragment shader inputs\n- How textures are sampled and interpolated across triangles\n- **Pixel alignment considerations** for arbitrary image dimensions\n- **Coordinate system differences** between image formats and OpenGL","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-key-learning-points"},{"type":"module","title":"State Management Evolution - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"This example marks change in the LearnOpenGL series implementation.\nStarting with textures, our OpenGL state becomes more complex as we need to track\nmultiple resources: shader programs, vertex arrays, buffers, and now texture objects.\n\n**From 4.1 forward**, we transition to map-based state management for better\nmaintainability and readability as examples grow in complexity.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-state-management-evolution"},{"type":"module","title":"Vertex Data Structure - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"The rectangle uses interleaved vertex data:\n- Position (3 floats): x, y, z coordinates\n- Color (3 floats): r, g, b values (for potential mixing)\n- Texture coordinates (2 floats): s, t coordinates","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-vertex-data-structure"},{"type":"module","title":"Texture Mapping - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"The rectangle is mapped with texture coordinates:\n- Bottom-left: (0.0, 0.0)\n- Bottom-right: (1.0, 0.0)\n- Top-right: (1.0, 1.0)\n- Top-left: (0.0, 1.0)","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-texture-mapping"},{"type":"module","title":"Dependencies - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"- **Optional**: `{:stb_image, \"~> 0.6\"}` for real image loading\n- **Fallback**: Procedural checkerboard generation when stb_image unavailable\n- **Educational**: Works in both scenarios with helpful guidance","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-dependencies"},{"type":"module","title":"Usage - EAGL.Examples.LearnOpenGL.GettingStarted.Textures","doc":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.run_example()\n\nPress ENTER to exit the example.","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#module-usage"},{"type":"function","title":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.run_example/0","doc":"","ref":"EAGL.Examples.LearnOpenGL.GettingStarted.Textures.html#run_example/0"},{"type":"extras","title":"README","doc":"EAGL \n   \n   \n    Make it EAsier to work \n    with OpenGL in Elixir.","ref":"readme.html"},{"type":"extras","title":"Overview - README","doc":"Most examples of working with OpenGL are written in C++ or C# (Unity). The purpose of the EAGL library is to:\n- Make it easier to translate OpenGL tutorials and examples from resources like [Learn OpenGL](https://learnopengl.com) into Elixir. \n- Provide basic helper functions to bridge the gap between idiomatic Elixir and OpenGL's state machine, using the Wings 3D Erlang source as a guide to prescriptive vs helpful additions\n- Enable other libraries and apps to build on this one and libraries like [ECSx](https://github.com/ecsx-framework/ECSx) and the list at [Awesome Elixir Gaming](https://github.com/njwest/Awesome-Elixir-Gaming)\n\nThe following are non-goals:\n- Focussing on 2D GPU graphics (see [Scenic](https://github.com/ScenicFramework/scenic) for that)\n- Wrapping of the Erlang wx library\n- A Shader DSL\n- A UI layout/component library\n- 3D mesh modelling (leave that to Wings 3D, Blender etc)","ref":"readme.html#overview"},{"type":"extras","title":"Quick Start - README","doc":"```elixir\n# Add to mix.exs\n{:eagl, \"~> 0.5.0\"}\n```\n\nEAGL includes several examples to demonstrate its capabilities. Use the unified examples runner:\n\n```\n./priv/scripts/run_examples\n════════════════════════════════════════════════════════════════\n                         EAGL Examples Menu\n════════════════════════════════════════════════════════════════\n\nNon-LearnOpenGL Examples:\n  01) Math Example - Comprehensive EAGL.Math functionality demo\n  02) Teapot Example - 3D teapot with Phong shading\n\nLearnOpenGL Getting Started Examples:\n\n  Hello Window:     111) 1.1 Window    112) 1.2 Clear Colors\n\n  Hello Triangle:   121) 2.1 Triangle  122) 2.2 Indexed    123) 2.3 Exercise1\n                    124) 2.4 Exercise2 125) 2.5 Exercise3\n\n  Shaders:          131) 3.1 Uniform   132) 3.2 Interpolation 133) 3.3 Class\n                    134) 3.4 Exercise1 135) 3.5 Exercise2     136) 3.6 Exercise3\n\n  Textures:         141) 4.1 Basic     142) 4.2 Combined      143) 4.3 Exercise1\n                    144) 4.4 Exercise2 145) 4.5 Exercise3     146) 4.6 Exercise4\n\n  Missing:          151) 5.1 Transformations  161) 6.1 Coordinate Systems  171) 7.1 Camera\n\n════════════════════════════════════════════════════════════════\nEnter code (01, 02, 111-146), 'q' to quit, 'r' to refresh:\n>\n```","ref":"readme.html#quick-start"},{"type":"extras","title":"Usage - README","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Math Operations - README","doc":"EAGL provides a comprehensive 3D math library based on GLM supporting:\n\n- **Vectors**: 2D, 3D, 4D vector operations with constructor macros\n- **Matrices**: 2x2, 3x3, 4x4 matrix operations with transformation functions\n- **Quaternions**: Rotation representation, SLERP, and conversion functions\n- **Utilities**: Trigonometry, interpolation, clamping, and geometric functions\n- **OpenGL Integration**: All functions work with the tuple-in-list format required by Erlang's OpenGL bindings\n- **Sigils**: Compile-time validated literals for matrices (`~m`), vertices (`~v`), and indices (`~i`)\n\n#### Sigil Literals\n\nEAGL provides three sigils for creating OpenGL data with compile-time validation and clean tabular formatting:\n\n```elixir\nimport EAGL.Math\n\n# Matrix sigil (~m) - supports comments and automatic size detection\nidentity_4x4 = ~m\"\"\"\n1.0  0.0  0.0  0.0\n0.0  1.0  0.0  0.0\n0.0  0.0  1.0  0.0\n0.0  0.0  0.0  1.0\n\"\"\"\n\ntransform_matrix = ~m\"\"\"\n1.0  0.0  0.0  10.0  # Translation X\n0.0  1.0  0.0  20.0  # Translation Y\n0.0  0.0  1.0  30.0  # Translation Z\n0.0  0.0  0.0   1.0\n\"\"\"\n\n# Vertex sigil (~v) - for raw vertex buffer data\ntriangle_vertices = ~v\"\"\"\n# position      color\n 0.0   0.5  0.0  1.0  0.0  0.0  # top vertex - red\n-0.5  -0.5  0.0  0.0  1.0  0.0  # bottom left - green\n 0.5  -0.5  0.0  0.0  0.0  1.0  # bottom right - blue\n\"\"\"\n\n# Index sigil (~i) - for element indices (must be integers)\nquad_indices = ~i\"\"\"\n0  1  3  # first triangle\n1  2  3  # second triangle\n\"\"\"\n```\n\n#### Vector and Matrix Operations\n\n```elixir\nimport EAGL.Math\n\n# Vector operations\nposition = vec3(1.0, 2.0, 3.0)\ndirection = vec3(0.0, 1.0, 0.0) \nresult = vec_add(position, direction)\nlength = vec_length(position)\n\n# Matrix transformations\nmodel = mat4_translate(vec3(5.0, 0.0, 0.0))\nview = mat4_look_at(\n  vec3(0.0, 0.0, 5.0),  # eye\n  vec3(0.0, 0.0, 0.0),  # target\n  vec3(0.0, 1.0, 0.0)   # up\n)\nprojection = mat4_perspective(radians(45.0), 16.0/9.0, 0.1, 100.0)\n```","ref":"readme.html#math-operations"},{"type":"extras","title":"Shader Management - README","doc":"The uniform helpers (from Wings3D) automatically detect the type of EAGL.Math values, eliminating the need to manually unpack vectors or handle different uniform types:\n\n- `vec2/3/4` → `glUniform2f/3f/4f`\n- `mat2/3/4` → `glUniformMatrix2fv/3fv/4fv` \n- Numbers → `glUniform1f/1i`\n- Booleans → `glUniform1i` (0 or 1)\n\n```elixir\nimport EAGL.Shader\n\n# Compile and link shaders\n{:ok, vertex} = create_shader(:vertex, \"vertex.glsl\")\n{:ok, fragment} = create_shader(:fragment, \"fragment.glsl\")\n{:ok, program} = create_attach_link([vertex, fragment])\n\n# Set uniforms with automatic type detection\nset_uniform(program, \"model_matrix\", model_matrix)\nset_uniform(program, \"light_position\", vec3(10.0, 10.0, 5.0))\nset_uniform(program, \"time\", :erlang.monotonic_time(:millisecond))\n\n# Or set multiple uniforms at once\nset_uniforms(program, [\n  model: model_matrix,\n  view: view_matrix,\n  projection: projection_matrix,\n  light_position: vec3(10.0, 10.0, 5.0),\n  light_color: vec3(1.0, 1.0, 1.0)\n])\n```","ref":"readme.html#shader-management"},{"type":"extras","title":"Texture Management - README","doc":"EAGL provides meaningful texture abstractions:\n- **Image Loading**: `load_texture_from_file()` with automatic fallback to checkerboard patterns\n- **Texture Creation**: `create_texture()` returns `{:ok, id}` tuples for error handling\n- **Parameter Setting**: `set_texture_parameters()` converts atoms to OpenGL constants\n- **Data Loading**: `load_texture_data()` handles format/type conversion with defaults\n- **Procedural Textures**: `create_checkerboard_texture()` generates test patterns\n- **Graceful Degradation**: Helpful warnings when optional dependencies aren't available\n- **Direct OpenGL**: Use `:gl` functions directly for binding, mipmaps, and cleanup\n\n\n```elixir\nimport EAGL.Texture\nimport EAGL.Error\n\n# Load texture from image file (requires optional stb_image dependency)\n{:ok, texture_id, width, height} = load_texture_from_file(\"priv/images/eagl_logo_black_on_white.jpg\")\n\n# Or create procedural textures for testing\n{:ok, texture_id, width, height} = create_checkerboard_texture(256, 32)\n\n# Manual texture creation and configuration\n{:ok, texture_id} = create_texture()\n:gl.bindTexture(@gl_texture_2d, texture_id)\n\n# Set texture parameters with atom-to-constant conversion\nset_texture_parameters(\n  wrap_s: :repeat,\n  wrap_t: :repeat,\n  min_filter: :linear_mipmap_linear,\n  mag_filter: :linear\n)\n\n# Load pixel data with format handling\nload_texture_data(width, height, pixel_data, \n  internal_format: :rgb,\n  format: :rgb,\n  type: :unsigned_byte\n)\n\n# Generate mipmaps and check for errors\n:gl.generateMipmap(@gl_texture_2d)\ncheck(\"After generating mipmaps\")\n\n# Use multiple textures\n:gl.activeTexture(@gl_texture0)\n:gl.bindTexture(@gl_texture_2d, texture1_id)\n:gl.activeTexture(@gl_texture1)\n:gl.bindTexture(@gl_texture_2d, texture2_id)\n\n# Clean up\n:gl.deleteTextures([texture_id])\n```","ref":"readme.html#texture-management"},{"type":"extras","title":"Model Loading - README","doc":"Currently we only support the .obj format.\n\n```elixir\nimport EAGL.Model\n\n# Load OBJ file (with automatic normal generation if missing)\n{:ok, model} = load_model_to_vao(\"teapot.obj\")\n\n# Render the model\n:gl.bindVertexArray(model.vao)\n:gl.drawElements(@gl_triangles, model.vertex_count, @gl_unsigned_int, 0)\n```","ref":"readme.html#model-loading"},{"type":"extras","title":"Buffer Management - README","doc":"EAGL provides type-safe, buffer management with automatic stride/offset calculation and standard attribute helpers.\n\n```elixir\nimport EAGL.Buffer\n\n# Simple position-only VAO/VBO (most common case)\nvertices = ~v\"\"\"\n-0.5  -0.5  0.0\n 0.5  -0.5  0.0\n 0.0   0.5  0.0\n\"\"\"\n{vao, vbo} = create_position_array(vertices)\n\n# Multiple attribute configuration - choose your approach:\n# Position + color vertices (6 floats per vertex: x,y,z,r,g,b)\nposition_color_vertices = ~v\"\"\"\n# position      color\n-0.5  -0.5  0.0  1.0  0.0  0.0  # vertex 1: position + red\n 0.5  -0.5  0.0  0.0  1.0  0.0  # vertex 2: position + green  \n 0.0   0.5  0.0  0.0  0.0  1.0  # vertex 3: position + blue\n\"\"\"\n\n# APPROACH 1: Automatic calculation (recommended for standard layouts)\n# Automatically calculates stride/offset - no manual math required.\nattributes = vertex_attributes(:position, :color)\n{vao, vbo} = create_vertex_array(position_color_vertices, attributes)\n\n# APPROACH 2: Manual configuration (for fine control or non-standard layouts)  \n# Specify exactly what you want - useful for custom stride, non-sequential locations, etc.\nattributes = [\n  position_attribute(),                           # location: 0, size: 3, stride: 24, offset: 0\n  color_attribute(stride: 24, offset: 12)         # location: 1, size: 3, stride: 24, offset: 12\n]\n{vao, vbo} = create_vertex_array(position_color_vertices, attributes)\n\n# Use automatic approach when:  - Standard position/color/texture/normal layouts\n#                               - Sequential attribute locations (0, 1, 2, 3...)\n#                               - Tightly packed (no padding between attributes)\n#\n# Use manual approach when:     - Custom attribute locations or sizes\n#                               - Non-standard data types or normalization \n#                               - Attribute padding or unusual stride patterns\n\n# Indexed geometry (rectangles, quads, models)\nquad_vertices = ~v\"\"\"\n 0.5   0.5  0.0  # top right\n 0.5  -0.5  0.0  # bottom right\n-0.5  -0.5  0.0  # bottom left\n-0.5   0.5  0.0  # top left\n\"\"\"\nindices = ~i\"\"\"\n0  1  3  # first triangle\n1  2  3  # second triangle\n\"\"\"\n{vao, vbo, ebo} = create_indexed_position_array(quad_vertices, indices)\n\n# Complex interleaved vertex data with multiple attributes\n# Format: position(3) + color(3) + texture_coord(2) = 8 floats per vertex\ninterleaved_vertices = ~v\"\"\"\n# x     y     z     r     g     b     s     t\n-0.5  -0.5   0.0   1.0   0.0   0.0   0.0   0.0  # bottom left\n 0.5  -0.5   0.0   0.0   1.0   0.0   1.0   0.0  # bottom right\n 0.0   0.5   0.0   0.0   0.0   1.0   0.5   1.0  # top centre\n\"\"\"\n\n# Three standard attributes with automatic calculation\n{vao, vbo} = create_vertex_array(interleaved_vertices, vertex_attributes(:position, :color, :texture_coordinate))\n\n# Clean up resources\ndelete_vertex_array(vao, vbo)\ndelete_indexed_array(vao, vbo, ebo)  # For indexed arrays\n```\n\n**Standard Attribute Helpers:**\n- `position_attribute()` - 3 floats (x, y, z) at location 0\n- `color_attribute()` - 3 floats (r, g, b) at location 1  \n- `texture_coordinate_attribute()` - 2 floats (s, t) at location 2\n- `normal_attribute()` - 3 floats (nx, ny, nz) at location 3\n\n**Key Benefits:**\n- **Automatic calculation**: `vertex_attributes()` eliminates manual stride/offset math\n- **Type safety**: Compile-time checks for attribute configuration  \n- **Standard patterns**: Common attribute layouts are pre-defined\n- **Flexible**: Mix automatic and manual configuration as needed","ref":"readme.html#buffer-management"},{"type":"extras","title":"Error Handling - README","doc":"```elixir\nimport EAGL.Error\n\n# Check for OpenGL errors with context\ncheck(\"After buffer creation\")  # Returns :ok or {:error, message}\n\n# Get human-readable error string for error code\nerror_string(1280)  # \"GL_INVALID_ENUM\"\n\n# Check and raise on error (useful for debugging)\ncheck!(\"Critical operation\")  # Raises RuntimeError if error found\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Window Creation - README","doc":"EAGL provides flexible window creation with a clean, options-based API:\n\n- **Default Size**: 1024x768 pixels (can be customized with `size:` option)\n- **2D Rendering** (default): No depth buffer, suitable for triangles, sprites, UI elements\n- **3D Rendering**: Enables depth testing and depth buffer for proper 3D scene rendering\n- **Automatic ENTER Handling**: Optional ENTER key handling for simple examples and tutorials\n- **Tick Events**: Automatic 60 FPS tick events for animations and updates (optional `handle_event/2` callback)\n\n```elixir\ndefmodule MyApp do\n  use EAGL.Window\n  import EAGL.Shader\n  import EAGL.Math\n\n  def run_example do\n    # For 2D rendering (triangles, sprites, UI) - uses default 1024x768 size\n    EAGL.Window.run(__MODULE__, \"My 2D OpenGL App\")\n    \n    # For 3D rendering (models, scenes with depth)\n    EAGL.Window.run(__MODULE__, \"My 3D OpenGL App\", depth_testing: true)\n    \n    # For tutorials/examples with automatic ENTER key handling\n    EAGL.Window.run(__MODULE__, \"Tutorial Example\", return_to_exit: true)\n    \n    # Custom window size and options\n    EAGL.Window.run(__MODULE__, \"Custom Size App\", size: {1280, 720}, depth_testing: true, return_to_exit: true)\n  end\n\n  @impl true\n  def setup do\n    # Initialize shaders, load models, etc.\n    {:ok, initial_state}\n  end\n\n  @impl true\n  def render(width, height, state) do\n    # Your render function should handle clearing the screen\n    :gl.clearColor(0.2, 0.3, 0.3, 1.0)\n    \n    # For 2D rendering (depth_testing: false, default)\n    :gl.clear(@gl_color_buffer_bit)\n    \n    # For 3D rendering (depth_testing: true)\n    # :gl.clear(@gl_color_buffer_bit ||| @gl_depth_buffer_bit)\n    \n    # Render your content here\n    :ok\n  end\n\n  @impl true\n  def cleanup(state) do\n    # Clean up resources\n    :ok\n  end\n\n  # Optional: Handle tick events for animations (60 FPS)\n  @impl true\n  def handle_event(:tick, state) do\n    # Update animations, physics, etc.\n    {:ok, updated_state}\n  end\nend\n```","ref":"readme.html#window-creation"},{"type":"extras","title":"Requirements - README","doc":"- **Elixir**: 1.14 or later\n- **Erlang/OTP**: 25 or later (with wx support - included in standard distributions)\n- **OpenGL**: 3.3 or later (for modern shader support)","ref":"readme.html#requirements"},{"type":"extras","title":"Platform-specific Notes - README","doc":"#### All Platforms\nEAGL uses Erlang's built-in `wx` module for windowing, which is included with standard Erlang/OTP installations. No additional GUI libraries need to be installed.\n\n#### Linux\nEnsure you have OpenGL drivers installed:\n```bash\n# Ubuntu/Debian\nsudo apt-get install libgl1-mesa-dev libglu1-mesa-dev\n\n# Fedora/RHEL\nsudo dnf install mesa-libGL-devel mesa-libGLU-devel\n```\n\n#### macOS\nOpenGL is included with macOS. No additional setup required.\n\n**Note**: EAGL automatically detects macOS and enables forward compatibility for OpenGL 3.0+ contexts, which is required by Apple's OpenGL implementation. This matches the behaviour of the `#ifdef __APPLE__` code commonly found in OpenGL tutorials.\n\n#### Windows  \nOpenGL is typically available through graphics drivers. If you encounter issues, ensure your graphics drivers are up to date.","ref":"readme.html#platform-specific-notes"},{"type":"extras","title":"Installation - README","doc":"1. Clone the repository:\n   ```bash\n   git clone https://github.com/yourusername/eagl.git\n   cd eagl\n   ```\n\n2. Install dependencies:\n   ```bash\n   mix deps.get\n   ```\n\n3. Compile the project:\n   ```bash\n   mix compile\n   ```\n\n4. Run tests to verify everything works:\n   ```bash\n   mix test\n   ```\n\n5. Try the examples:\n   ```bash\n   ./priv/scripts/run_examples\n   ```","ref":"readme.html#installation"},{"type":"extras","title":"Project Structure - README","doc":"```\nlib/\n├── eagl/                   # Core EAGL modules\n│   ├── buffer.ex           # VAO/VBO helper functions\n│   ├── const.ex            # OpenGL constants\n│   ├── error.ex            # Error checking and reporting\n│   ├── math.ex             # GLM-style math library\n│   ├── model.ex            # 3D model management\n│   ├── obj_loader.ex       # Wavefront OBJ parser\n│   ├── shader.ex           # Shader compilation\n│   ├── window.ex           # Window management\n│   └── window_behaviour.ex # Window callback behavior\n├── examples/               # Example applications\n│   ├── math_example.ex     # Math library demonstrations\n│   ├── teapot_example.ex   # 3D teapot rendering\n│   └── learnopengl/        # LearnOpenGL tutorial ports\n└── wx/                     # wxWidgets constants\ntest/\n├── eagl/                   # Unit tests for EAGL modules\n└── eagl_test.exs           # Integration tests\npriv/\n├── models/                 # 3D model files (.obj)\n├── scripts/                # Convenience scripts\n│   └── run_examples        # Unified examples runner\n└── shaders/                # GLSL shader files\n    └── learnopengl/        # LearnOpenGL tutorial shaders\n```","ref":"readme.html#project-structure"},{"type":"extras","title":"Features - README","doc":"- ✅ **Shader Management**: Automatic compilation, linking, and error reporting\n- ✅ **Texture Management**: Comprehensive texture creation, configuration, and loading\n- ✅ **3D Model Loading**: Wavefront OBJ format with normals and texture coordinates\n- ✅ **Math Library**: GLM-compatible vectors, matrices, quaternions with full OpenGL integration\n- ✅ **Buffer Helpers**: Wings3D-inspired VAO/VBO management functions\n- ✅ **Error Handling**: Comprehensive OpenGL error checking and reporting\n- ✅ **Window Management**: Cross-platform window creation with wxWidgets\n- ✅ **Event Handling**: Resize, close, paint, and 60 FPS tick events\n- ✅ **Resource Cleanup**: Automatic cleanup of OpenGL resources\n- ✅ **LearnOpenGL Examples**: Partial \"Getting Started\" series - direct ports of OpenGL tutorials\n- ✅ **Testing**: Full test suite with OpenGL context mocking","ref":"readme.html#features"},{"type":"extras","title":"Roadmap - README","doc":"The current focus is to:\n\n- [ ] **In Progress**: Complete the \"Getting Started\" LearnOpenGL examples series\n  - ✅ Hello Window (1.1-1.2): 2 examples\n  - ✅ Hello Triangle (2.1-2.5): 5 examples  \n  - ✅ Shaders (3.1-3.6): 6 examples\n  - ✅ Textures (4.1-4.6): 6 examples\n  - [ ] **Missing**: Transformations (5.1): 1 example needed\n  - [ ] **Missing**: Coordinate Systems (6.1): 1 example needed  \n  - [ ] **Missing**: Camera (7.1): 1 example needed\n- [ ] Continue with \"Lighting\" chapter examples\n- [ ] Load common model types like GLTF\n\nAnd in future:\n\n- [ ] Be able to apply post-processing effects\n- [ ] More extensive camera/lighting/material helpers\n- [ ] Access to a physics engine\n- [ ] Built-in GPU profiling tools","ref":"readme.html#roadmap"},{"type":"extras","title":"Troubleshooting - README","doc":"","ref":"readme.html#troubleshooting"},{"type":"extras","title":"Common Issues - README","doc":"#### Interactive Examples Not Responding\nExamples require user interaction (ENTER key to exit). This can cause issues during testing:\n\n```bash\n# Run only unit tests, excluding interactive examples\nmix test test/eagl/ --exclude interactive\n\n# Set a timeout for interactive tests\nmix test --timeout 10000\n```\n\n#### IEx Break Prompt\nIf you encounter an unexpected error in IEx and see a `BREAK: (a)bort` prompt, this indicates a crash in the BEAM VM. Enter 'a' to abort and return to the shell, then investigate the error that caused the crash.\n\n#### Test Timeouts in CI\nInteractive examples wait for user input and will timeout in continuous integration:\n- Examples are tagged with `@tag :interactive`\n- CI environments automatically exclude these tests\n- Run interactive tests individually during local development","ref":"readme.html#common-issues"},{"type":"extras","title":"Platform-Specific Issues - README","doc":"#### OpenGL Context Creation Failures\nIf you encounter context creation errors:\n- **Linux**: Ensure mesa development packages are installed\n- **macOS**: Update to a supported macOS version (10.9+)\n- **Windows**: Update graphics drivers\n\n#### Missing Dependencies\nIf optional dependencies are missing, EAGL will show warnings but continue with fallback behaviour:\n- Image loading falls back to procedural textures\n- Missing models show error messages but don't crash","ref":"readme.html#platform-specific-issues"},{"type":"extras","title":"Contributing - README","doc":"We welcome contributions. Suggested contributions include:\n- **LearnOpenGL tutorial ports**: Help complete the tutorial series\n- **Documentation improvements**: Examples, tutorials, API documentation\n- **Platform-specific optimisations**: Performance or compatibility improvements\n- **Example applications**: Links to demo projects showcasing EAGL capabilities\n- **Bug fixes**: Issues with existing functionality\n- **Testing improvements**: Better mocks, integration tests, or test utilities\n\nPlease read through these guidelines before submitting changes.","ref":"readme.html#contributing"},{"type":"extras","title":"Development Setup - README","doc":"1. Fork and clone the repository\n2. Install dependencies: `mix deps.get`\n3. Run tests to ensure everything works: `mix test`\n4. Try the examples: `./priv/scripts/run_examples`","ref":"readme.html#development-setup"},{"type":"extras","title":"Code Standards - README","doc":"#### Style Guidelines\n- Follow standard Elixir formatting (`mix format`) except keep matricies in tabular format\n- Use descriptive variable names, especially for OpenGL state\n- Include typespecs for public functions\n- Document complex algorithms and OpenGL-specific concepts\n\n#### Testing Requirements  \n- Add tests for new functionality\n- Ensure existing tests pass: `mix test`\n- Tag interactive tests with `@tag :interactive`\n- Mock OpenGL calls in unit tests where possible\n\n#### Documentation Standards\n- Update README.md for new features\n- Add docstrings for public functions\n- Include code examples in documentation\n- Our tone is calm, concise and factual e.g. avoid 'sales' language and over-use of '!'\n- Write in Australian/British English for documentation, US English for code","ref":"readme.html#code-standards"},{"type":"extras","title":"Design Philosophy - README","doc":"EAGL focuses on **meaningful abstractions** rather than thin wrappers around OpenGL calls:\n\n#### ✅ **Provide Value**\n- **Error handling**: `{:ok, result}` tuples and comprehensive error checking\n- **Type safety**: Atoms to OpenGL constants (`wrap_s: :repeat`)\n- **Sensible defaults**: Reduce boilerplate with common parameter combinations\n- **Complex operations**: Multi-step procedures like shader compilation and linking\n- **Data transformations**: Converting Elixir structures to OpenGL formats\n- **Testing utilities**: Procedural textures and geometry for development\n\n#### ❌ **Avoid Thin Wrappers**\n- **Simple OpenGL calls**: Use `:gl.bindTexture()`, `:gl.generateMipmap()` directly\n- **One-line functions**: Don't wrap functions that only add `check()` calls\n- **State management**: Let users manage OpenGL state explicitly when appropriate\n\n#### 🎯 **User Experience Goals**\n- **Selective imports**: `import EAGL.Error` for explicit error checking\n- **Direct OpenGL access**: When EAGL doesn't add substantial value\n- **Direct OpenGL integration**: Mix EAGL helpers with direct OpenGL calls","ref":"readme.html#design-philosophy"},{"type":"extras","title":"Submitting Changes - README","doc":"1. **Create a feature branch**: `git checkout -b feature/descriptive-name`\n2. **Make your changes** following the style guidelines above\n3. **Add or update tests** for your changes\n4. **Run the full test suite**: `mix test`\n5. **Update documentation** if you've added new features\n6. **Commit with clear messages**: Use present tense, describe what the commit does\n7. **Push your branch**: `git push origin feature/descriptive-name`\n8. **Open a Pull Request** with:\n   - Clear description of the changes\n   - Reference to any related issues\n   - Screenshots for visual changes\n   - Test results if applicable","ref":"readme.html#submitting-changes"},{"type":"extras","title":"Questions and Support - README","doc":"- **Issues**: Use GitHub issues for bugs and feature requests\n- **Discussions**: Use GitHub discussions for questions and design discussions\n- **Examples**: Look at existing code in `lib/examples/` for patterns","ref":"readme.html#questions-and-support"},{"type":"extras","title":"License - README","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Acknowledgments - README","doc":"- [Learn OpenGL](https://learnopengl.com) for excellent OpenGL book and tutorial code. If the examples helped you understand OpenGL better please consider a [donation to the author](https://www.paypal.me/learnopengl/), Joey De Vries.\n- [Wings3D](https://wings3d.com) for inspiration and helper function patterns - the name EAGL(e) is a tip of the hat to this project\n- The Erlang/OTP team and particularly Dan Gudmundsson for the wxWidgets bindings\n- The [local Elixir User Group](https://https://elixir.sydney) for putting up with my occasional random talks\n- [Cursor](https://cursor.com) and Claude Sonnet for giving me the patience to get to running code and porting Joey's Learning OpenGL examples","ref":"readme.html#acknowledgments"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 Robin Hiliard\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}